from wooly import *
from wooly.tables import *
from datetime import datetime

from util import *
from formats import *

strings = StringCatalog(__file__)

class ActionInvocationSet(ItemTable):
    def __init__(self, app, name):
        super(ActionInvocationSet, self).__init__(app, name)

        col = self.DescriptionColumn(app, "desc")
        self.add_column(col)

        col = self.WhenColumn(app, "when")
        self.add_column(col)

        col = self.StatusColumn(app, "status")
        self.add_column(col)

    class DescriptionColumn(ItemTableColumn):
        def render_title(self, session, item):
            return "Description"

        def render_content(self, session, item):
            return item.get_description(session)

    class WhenColumn(ItemTableColumn):
        def render_title(self, session, item):
            return "When"

        def render_content(self, session, item):
            delta = secs(datetime.now()) - secs(item.when)
            duration = fmt_duration(delta)
            if duration:
                return "%s ago" % duration
            else:
                ""

    class StatusColumn(ItemTableColumn):
        def render_title(self, session, item):
            return "Status"

        def render_content(self, session, item):
            if item.status == "pending":
                text = "Pending"
            elif item.status == "OK":
                text = "Completed"
            else:
                if item.exception:
                    text = "Failed: " + str(item.exception)
                else:
                    text = "Failed: " + item.status

            return text

    def do_get_items(self, session, *args):
        return reversed(sorted_by(self.app.model.invocations, "when"))

class ActionPage(Page):
    def __init__(self, app, name):
        super(ActionPage, self).__init__(app, name)

        self.__actions = ActionInvocationSet(app, "actions")
        self.add_child(self.__actions)

    def render_title(self, session, *args):
        return "Actions"
