from mint import *
from wooly import *
from wooly.widgets import *
from random import random

from virtualhost import *
from broker import *
from widgets import *
from parameters import *
from formats import *
from util import *

strings = StringCatalog(__file__)

class BrokerClusterSet(ItemSet):
    def render_cluster_add_href(self, session, *args):
        branch = session.branch()
        self.page.show_broker_cluster_add(branch)
        return branch.marshal()
    
    def render_title(self, session, *args):
        return "Broker Clusters %s" % fmt_count(BrokerCluster.select().count())

    def do_get_items(self, session, *args):
        return BrokerCluster.select()

    def render_item_link(self, session, cluster):
        branch = session.branch()
        self.page.show_broker_cluster(branch, cluster).show_view(branch)
        return fmt_olink(branch, cluster)

    def render_item_config(self, session, cluster):
        return len(cluster.brokers)

    def render_item_status(self, session, cluster):
        writer = Writer()
        
        for broker in sorted_by(cluster.brokers):
            writer.write(fmt_ostatus(broker))

        return writer.to_string()

class BrokerClusterFrame(CuminFrame):
    def __init__(self, app, name):
        super(BrokerClusterFrame, self).__init__(app, name)

        self.object = BrokerClusterParameter(app, "id")
        self.add_parameter(self.object)

        view = BrokerClusterView(app, "view")
        self.add_mode(view)
        self.set_view_mode(view)

        edit = BrokerClusterEdit(app, "edit")
        self.add_mode(edit)
        self.set_edit_mode(edit)

        remove = BrokerClusterRemove(app, "remove")
        self.add_mode(remove)
        self.set_remove_mode(remove)

        self.broker = BrokerFrame(app, "broker")
        self.add_mode(self.broker)

    def show_broker(self, session, broker):
        self.broker.set_object(session, broker)
        return self.show_mode(session, self.broker)

class BrokerClusterStatus(CuminStatus):
    pass

class BrokerClusterView(CuminView):
    def __init__(self, app, name):
        super(BrokerClusterView, self).__init__(app, name)

        self.status = BrokerClusterStatus(app, "status")
        self.add_child(self.status)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)

        self.tabs.add_tab(self.ClusterBrokerTab(app, "brokers"))
        self.tabs.add_tab(self.ClusterStatsTab(app, "stats"))

    class ClusterBrokerTab(BrokerSet):
        def get_args(self, session):
            return self.frame.get_args(session)

        def render_title(self, session, cluster):
            return "Brokers %s" % fmt_count(len(cluster.brokers))

        def do_get_items(self, session, cluster):
            return cluster.brokers

    class ClusterStatsTab(Widget):
        def get_args(self, session):
            return self.frame.get_args(session)

        def render_title(self, session, cluster):
            return "Statistics"

class BrokerClusterForm(CuminForm):
    def __init__(self, app, name):
        super(BrokerClusterForm, self).__init__(app, name)

        self.cluster_name = StringInput(app, "name")
        self.add_child(self.cluster_name)

    def process_cluster(self, session, cluster):
        cluster.name = self.cluster_name.get(session)

        branch = session.branch()
        self.page.show_broker_cluster(branch, cluster).show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

class BrokerClusterAdd(BrokerClusterForm, Frame):
    def render_title(self, session):
        return "Add Cluster"

    def process_cancel(self, session):
        branch = session.branch()
        self.page.main.show(branch)
        self.page.set_redirect_url(session, branch.marshal())
    
    def process_submit(self, session):
        cluster = BrokerCluster()
        self.process_cluster(session, cluster)
    
class BrokerClusterEdit(BrokerClusterForm, Frame):
    def get_args(self, session):
        return self.frame.get_args(session)

    def process_cancel(self, session, cluster):
        branch = session.branch()
        self.parent.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, cluster):
        self.process_cluster(session, cluster)

    def process_display(self, session, cluster):
        self.cluster_name.set(session, cluster.name)

    def render_title(self, session, cluster):
        return "Edit Cluster '%s'" % cluster.name

class BrokerClusterRemove(CuminConfirmForm):
    def get_args(self, session):
        return self.get_args(session)

    def process_cancel(self, session, cluster):
        branch = session.branch()
        self.page.show_broker_cluster(branch, cluster).show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, cluster):
        cluster.destroySelf()

        branch = session.branch()
        self.page.main.show(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def render_title(self, session, cluster):
        return "Remove Broker Cluster '%s'" % cluster.name

    def render_submit_content(self, session, cluster):
        return "Yes, Remove Broker Cluster '%s'" % cluster.name

    def render_cancel_content(self, session, cluster):
        return "No, Cancel"
