from mint import *
from wooly import *
from wooly.widgets import *

from queue import *
from exchange import *
from widgets import *
from parameters import *
from formats import *
from util import *

strings = StringCatalog(__file__)

class PeerSet(CuminTable, Form):
    def __init__(self, app, name):
        super(PeerSet, self).__init__(app, name)

        self.ids = FilteredCheckboxIdColumn(app, "id", self, callback=self.disable_closed)
        self.add_column(self.ids)

        #self.ids = CheckboxIdColumn(app, "id")
        #self.add_column(self.ids)

        col = self.AddressColumn(app, "addr")
        self.add_column(col)
        self.set_default_column(col)

        col = self.DurableColumn(app, "durable")
        self.add_column(col)

        col = self.StateColumn(app, "state")
        self.add_column(col)

        col = self.LastErrorColumn(app, "last_error")
        self.add_column(col)

        #col = self.FromPeerColumn(app, "from_peer")
        #self.add_column(col)

        #col = self.ToPeerColumn(app, "to_peer")
        #self.add_column(col)
        
        self.__close = self.Close(app, "close")
        self.add_child(self.__close)

    def render_add_broker_link_url(self, session, vhost):
        branch = session.branch()
        self.frame.link_add.show(branch)
        return branch.marshal()

    def get_args(self, session):
        reg = self.frame.get_object(session)
        return (reg.getDefaultVhost(),)

    def render_title(self, session, vhost):
        count = self.get_item_count(session, vhost)
        return "Broker Links %s" % fmt_count(count)

    def render_sql_where(self, session, vhost):
        return "where v.id = %(vhost_id)r"

    def get_sql_values(self, session, vhost):
        return {"vhost_id": vhost.id}

    def disable_closed(self, session, data):
        return data["state"] == "Closed"
        
    class AddressColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Address"

        def render_content(self, session, data):
            peer = Identifiable(data["id"])
            href = self.frame.link.get_href(session, peer)
            name = "%s:%i" % (data["host"], data["port"])
            return fmt_link(href, fmt_shorten(name))

        def get_orderby_sql(self, session):
            dir = self.parent.is_reversed(session) and "desc" or "asc"
            return "order by host, port %s" % dir

    class StateColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "State"

    class LastErrorColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Last Error"
        
    class DurableColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Durable"

    class FromPeerColumn(NullSortColumn, FreshDataOnlyColumn):
        def render_title(self, session, data):
            return "Bytes from Broker"

        def render_value(self, session, value):
            return fmt_rate(value)

    class ToPeerColumn(NullSortColumn, FreshDataOnlyColumn):
        def render_title(self, session, data):
            return "Bytes to Broker"

        def render_value(self, session, value):
            return fmt_rate(value)

    class Close(FormButton):
        def render_content(self, session):
            return "Close"

        def process_submit(self, session):
            ids = self.parent.ids.get(session)
            self.parent.ids.clear(session)

            href = self.frame.links_close.get_href(session, ids)
            self.page.set_redirect_url(session, href)

class PeerRouteSet(CuminTable, Form):
    def __init__(self, app, name):
        super(PeerRouteSet, self).__init__(app, name)

        self.ids = CheckboxIdColumn(app, "id")
        self.add_column(self.ids)

        #col = self.SourceColumn(app, "source")
        #self.add_column(col)

        #col = self.DestinationColumn(app, "managed_broker")
        #self.add_column(col)

        col = self.ExchangeColumn(app, "exchange")
        self.add_column(col)
        self.set_default_column(col)

        col = self.KeyColumn(app, "key")
        self.add_column(col)
        
        col = self.TagColumn(app, "tag")
        self.add_column(col)
        
        col = self.ExcludesColumn(app, "excludes")
        self.add_column(col)

        self.__remove = self.Remove(app, "remove")
        self.add_child(self.__remove)

    def get_args(self, session):
        return self.frame.get_args(session)

    def render_title(self, session, link):
        count = self.get_item_count(session, link)
        return "Link Routes %s" % fmt_count(count)

    def render_sql_where(self, session, link):
        return "where l.id = %(link_id)r and b.qmf_delete_time is null"
    
    def get_sql_values(self, session, link):
        return {"link_id": link.id}

    def render_add_bridge_url(self, session, route):
        branch = session.branch()
        self.frame.show_bridge_add(branch)
        return branch.marshal()
        
    class SourceColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Source"

        def render_content(self, session, data):
            return "%s:%i" % (data["host"], data["port"])

        def get_orderby_sql(self, session):
            dir = self.parent.is_reversed(session) and "desc" or "asc"
            return "order by l.host, l.port %s" % dir

    class DestinationColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Destination"

    class ExchangeColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Exchange"

    class KeyColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Routing Key"

    class TagColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Tag"

    class ExcludesColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Excludes"

    class Remove(FormButton):
        def render_content(self, session):
            return "Remove"

        def process_submit(self, session):
            ids = self.parent.ids.get(session)
            self.parent.ids.clear(session)

            branch = session.branch()
            frame = self.frame.show_routes_close(branch)
            frame.ids.set(branch, ids)
            self.page.set_redirect_url(session, branch.marshal())

class PeerFrame(CuminFrame):
    def __init__(self, app, name):
        super(PeerFrame, self).__init__(app, name)

        self.object = PeerParameter(app, "id")
        self.add_parameter(self.object)

        self.view = PeerView(app, "view")
        self.add_mode(self.view)
        self.set_view_mode(self.view)

        remove = LinkRemove(app, "remove")
        self.add_mode(remove)
        self.set_remove_mode(remove)
        
        self.__bridge_add = BridgeAdd(app, "bridgeadd")
        self.add_mode(self.__bridge_add)
        
        self.__routes_close = PeerRouteSetClose(app, "routesclose")
        self.add_mode(self.__routes_close)
        
    def render_title(self, session, peer):
        return super(PeerFrame, self).render_title(session, peer)

    def show_object(self, session, peer):
        reg = peer.vhost.broker.registration
        self.page.main.broker.set_object(session, reg)

        return super(PeerFrame, self).show_object(session, peer)

    def show_bridge_add(self, session):
        self.page.set_current_frame(session, self.__bridge_add)
        return self.show_mode(session, self.__bridge_add) 

    def show_routes_close(self, session):
        self.page.set_current_frame(session, self.__routes_close)
        return self.show_mode(session, self.__routes_close)

class LinkRemove(CuminConfirmForm):
    def get_args(self, session):
        return self.frame.get_args(session)

    def process_cancel(self, session, link):
        branch = session.branch()
        self.frame.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, link):
        action = self.app.model.link.close
        action.invoke(link)

        branch = session.branch()
        self.frame.frame.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def render_title(self, session, link):
        return "Close Broker Link '%s:%d'" % (link.host, link.port)

    def render_submit_content(self, session, link):
        return "Yes, Close Broker Link"

    def render_cancel_content(self, session, link):
        return "No, Cancel"

class PeerStatus(CuminStatus):
    
    def render_color(self, session, link):
        if link.statsCurr:
            return link.statsCurr.lastError and "red" or "green"
    
    def render_peer_state(self, session, peer):
        if peer.statsCurr:
            return peer.statsCurr.state

    def render_peer_error(self, session, peer):
        if peer.statsCurr:
            return peer.statsCurr.lastError

class PeerView(CuminView):
    def __init__(self, app, name):
        super(PeerView, self).__init__(app, name)

        status = PeerStatus(app, "status")
        self.add_child(status)

        self.__tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.__tabs)

        #self.__tabs.add_tab(PeerStats(app, "stats"))

        self.__routes = PeerRouteSet(app, "routes")
        self.__tabs.add_tab(self.__routes)

        self.__tabs.add_tab(CuminDetails(app, "details"))

    def show_routes(self, session):
        return self.__tabs.show_mode(session, self.__routes);

    def render_script(self, session, peer):
        data = "model.xml?class=peer;id=%i" % peer.id
        return "wooly.setIntervalUpdate('%s', updatePeer, 3000)" % data

class PeerStats(TabbedModeSet):
    def __init__(self, app, name):
        super(PeerStats, self).__init__(app, name)

        self.add_child(StatSet(app, "io", "io"))
        self.add_child(StatSet(app, "general", "general"))

        chart = self.ReceiveRouteDropRateChart(app, "recvroutedrop")
        self.add_child(chart)

        chart = StatValueChart(app, "producers")
        chart.stats = ("producerCount",)
        self.add_child(chart)

    def render_title(self, session):
        return "Statistics"

    class ReceiveRouteDropRateChart(StatValueChart):
        def __init__(self, app, name):
            super(PeerStats.ReceiveRouteDropRateChart, self).__init__ \
                (app, name)

            self.stats = ("msgReceives", "msgRoutes", "msgDrops")
            self.mode = "rate"

        def render_title(self, session, exchange):
            return "Messages Received, Routed, and Dropped"

class ExchangeInputSet(RadioInputSet):
    def __init__(self, app, name, state):
        super(ExchangeInputSet, self).__init__(app, name, None)

        self.param = IntegerParameter(app, "param")
        self.add_parameter(self.param)

        self.state = state
        
    def do_get_items(self, session, *args):
        exchanges = list()
        
        link = self.frame.frame.get_object(session)
        vhost = link.vhost
        sortedExchanges = sorted_by(vhost.exchanges)
        
        for exchange in sortedExchanges:
            if ExchangeInfo.is_builtin(exchange) or \
               (not exchange._get_qmfDeleteTime() and \
                not (self.state.is_active(session) and not is_active(exchange))):
                if not exchange.name in ["qpid.management", ""]:
                    if not self.param.get(session):
                        self.param.set(session, exchange.id)
                    exchanges.append(exchange)
        
        return exchanges

    def render_item_value(self, session, exchange):
        return exchange.id

    def render_item_content(self, session, exchange):
        return exchange.name

    def render_item_checked_attr(self, session, exchange):
        if self.param.get(session) == exchange.id:
            return "checked=\"checked\""

class ExchangeRadioField(FormField):
    def __init__(self, app, name):
        super(ExchangeRadioField, self).__init__(app, name)

        self.state = ExchangeState(app, "phase")
        self.add_child(self.state)
        
        self.__exchanges = ExchangeInputSet(app, "inputs", self.state)
        self.add_child(self.__exchanges)
        
    def get(self, session):
        return self.__exchanges.get(session)
    
    def render_title(self, session):
        return "Choose an Exchange"


class BridgeAddForm(CuminFieldForm):
    def __init__(self, app, name):
        super(BridgeAddForm, self).__init__(app, name)

        self.exchange = ExchangeRadioField(app, "exchange")
        self.add_field(self.exchange)

        self.key = TextField(app, "key")
        self.key.set_title("Routing Key")
        self.add_field(self.key)

        self.more = MoreFieldSet(app, "more")
        self.add_field(self.more)
        
        self.help = self.BridgeAddHelpField(app, "help")
        self.more.add_field(self.help)
        
        self.tag = TextField(app, "tag")
        self.tag.set_required(False)
        self.tag.set_title("Tag")
        self.more.add_field(self.tag)

        self.excludes = TextField(app, "excludes")
        self.excludes.set_required(False)
        self.excludes.set_title("Excludes")
        self.more.add_field(self.excludes)

        self.dynamic = self.DynamicField(app, "dynamic")
        self.more.add_field(self.dynamic)

        self.sync = self.SyncField(app, "sync")
        self.more.add_field(self.sync)
        
    def get_args(self, session):
        return self.frame.get_args(session)
    
    def render_title(self, session, link):
        return "Add Route to '%s:%d'" % (link.host, link.port)
    
    def process_display(self, session, *args):
        if not self.tag.get(session):
            self.tag.set(session, args[0].qmfBrokerId)
        
        if not self.excludes.get(session):
            self.excludes.set(session, "%s:%s" % (args[0].host, args[0].port))
            
        if not self.sync.get(session):
            self.sync.set(session, self.sync.get_default(session))
            
    class SyncField(IntegerField):
        def render_title(self, session):
            return "Ack"
        
        def render_field_help(self, session):
             return "Acknowledge transfers over the bridge in batches of N"

        def get_default(self, session):
            return 0
        
    class DynamicField(TwoOptionRadioField):
        def render_title(self, session):
            return "Dynamic Route?"
        
        def render_field_help(self, session):
            return "(Should the added route be dynamic)"

        def render_title_1(self, session):
            return "Dynamic"
        
        def render_title_2(self, session):
            return "Not dynamic"
        
        
    class BridgeAddHelpField(FormField):
        pass

class BridgeAdd(BridgeAddForm):
    def process_cancel(self, session, *args):
        branch = session.branch()
        self.frame.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, link):
        errors = self.validate(session)
        if errors:
            pass
        else:
            key = self.key.get(session)
            tag = self.tag.get(session)
            excludes = self.excludes.get(session)
            exchange_id = self.exchange.get(session)
            exchange = Exchange.get(int(exchange_id))
            durable = link.durable
            dynamic = self.dynamic.get(session) == "yes"
            sync = self.sync.get(session)
            
            args = {"durable": durable,
                    "exchange": exchange.name,
                    "key": key,
                    "tag": tag,
                    "excludes": excludes,
                    "dynamic": dynamic,
                    "sync": sync
                    }
            
            action = self.app.model.link.bridge
            action.invoke(link, args)

            self.process_cancel(session, link)
            
class BrokerLinkAddForm(CuminFieldForm):
    def __init__(self, app, name):
        super(BrokerLinkAddForm, self).__init__(app, name)

        self.host = self.Host(app, "host")
        self.add_field(self.host)

        self.more = self.ShowButton(app, "more")
        self.add_field(self.more)
        
        self.port = self.PortField(app, "port")
        self.more.add_field(self.port)

        self.username = self.UsernameField(app, "username")
        self.more.add_field(self.username)

        self.password = self.PassField(app, "password")
        self.more.add_field(self.password)

        self.durable = self.DurableField(app, "durable")
        self.more.add_field(self.durable)
        
        self.transport = self.TransportField(app, "transport")
        self.more.add_field(self.transport)
                
    def render_title(self, session, *args):
        reg = self.frame.get_object(session)
        return "Add Broker Link to '%s'" % reg.name

    class ShowButton(MoreFieldSet):
        def render_more_text(self, session, *args):
            return "Show Optional Inputs..."
        
        def render_less_text(self, session, *args):
            return "Hide Optional Inputs..."
    
    class Host(NameField):
        def render_title(self, session):
            return "Address"

    class PortField(StringField):
        def __init__(self, app, name):
            super(BrokerLinkAddForm.PortField, self).__init__(app, name)
            
            self.input.size = 5
            self.css_class = "compact first"
            
        def render_title(self, session):
            return "Port"
        
        def do_validate(self, session, errors):
            try:
                port = self.get(session)
                if port:
                    int(port)
            except:
                errors.append(FormError("Port must be a number"))
    
    class UsernameField(StringField):
        def render_title(self, session):
            return "Username"

        def render_form_field_class(self, session):
            return "compact"
        
    class PassField(PasswordField):
        def render_title(self, session):
            return "Password"
        
        def render_form_field_class(self, session):
            return "compact last"
        
    class TransportField(RadioField):
        def __init__(self, app, name):
            super(BrokerLinkAddForm.TransportField, self).__init__(app, name, None)
    
            self.param = Parameter(app, "param")
            self.param.default = "tcp"
            self.add_parameter(self.param)
    
            option = self.TCP(app, "tcp", self.param)
            self.add_option(option)
            
            option = self.SSL(app, "ssl", self.param)
            self.add_option(option)
            
            option = self.RDMA(app, "rdma", self.param)
            self.add_option(option)
    
        def render_title(self, session):
            return "Transport-type"
    
        def render_field_help(self, session):
            return "(Transport to use)"
            
        class TCP(RadioFieldOption):
            def render_title(self, session):
                return "tcp"
    
        class SSL(RadioFieldOption):
            def render_title(self, session):
                return "ssl"
    
        class RDMA(RadioFieldOption):
            def render_title(self, session):
                return "rdma"
        
    class DurableField(TwoOptionRadioField):
        def render_title(self, session):
            return "Restore if broker restarts?"
        
        def render_field_help(self, session):
            return "(Should the added configuration be durable)"

        def render_title_1(self, session):
            return "Yes, restore if broker restarts"

        def render_title_2(self, session):
            return "No, do not restore if broker restarts"

class BrokerLinkAdd(BrokerLinkAddForm):
    def get_args(self, session):
        return (self.frame.get_object(session), )

    def process_cancel(self, session, *args):
        branch = session.branch()
        self.frame.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, reg):
        if self.validate(session):
            pass
        else:
            host = self.host.get(session)
            port = self.port.get(session) or 5672
            if port:
                port = int(port)
            username = self.username.get(session) or "anonymous"
            password = self.password.get(session)
            durable = self.durable.get(session)
            transport = self.transport.get(session)
            
            link = Link()
            link.host = host
            link.port = port
            link.durable = (durable == "yes")

            args = {"reg": reg,
                    "username": username,
                    "password": password,
                    "transport": transport}
            
            action = self.app.model.broker.add_link
            action.invoke(link, args)
            
            # navigate back to main queue frame
            self.process_cancel(session, (reg,))

class BrokerSetClose(CuminBulkActionForm):
    def process_return(self, session):
        branch = session.branch()
        self.frame.show_view(branch)
        self.page.set_current_frame(branch, self.frame)
        self.page.set_redirect_url(session, branch.marshal())

    def process_item(self, session, id):
        link = Link.get(id)
        action = self.app.model.link.close
        action.invoke(link)

    def render_title(self, session):
        return "Close Broker Link"
    
    def render_form_heading(self, session, *args):
        return "Close Link to"

    def render_item_content(self, session, id):
        link = Link.get(id)
        return "Broker: %s:%d" % (link.host, link.port)

class PeerRouteSetClose(CuminBulkActionForm):
    def process_return(self, session):
        branch = session.branch()
        self.frame.show_view(branch)
        self.page.set_current_frame(branch, self.frame)
        self.page.set_redirect_url(session, branch.marshal())

    def process_item(self, session, id):
        bridge = Bridge.get(id)
        action = self.app.model.route.remove
        action.invoke(bridge)

    def render_title(self, session):
        link = self.parent.get_object(session)
        return "Remove Route from %s:%i" % (link.host, link.port)
    
    def render_form_heading(self, session, *args):
        return "Remove Route"

    def render_item_content(self, session, id):
        bridge = Bridge.get(id)
        return "<td>%s</td><td>%s</td>" % (bridge.dest, bridge.key)

       
