from mint import *
from wooly import *
from wooly.widgets import *

from configproperty import *
from broker import *
from widgets import *
from parameters import *
from formats import *
from util import *

strings = StringCatalog(__file__)

class BrokerProfileSet(object):
    def render_profile_add_href(self, session, *args):
        branch = session.branch()
        self.page.show_broker_profile_add(branch)
        return branch.marshal()

    def render_title(self, session, *args):
        return "Broker Profiles %s" % fmt_count(BrokerProfile.select().count())

    def do_get_items(self, session, *args):
        return BrokerProfile.select()

    def render_item_link(self, session, profile):
        branch = session.branch()
        self.page.show_broker_profile(branch, profile).show_view(branch)
        return fmt_olink(branch, profile)

class BrokerProfileFrame(CuminFrame):
    def __init__(self, app, name):
        super(BrokerProfileFrame, self).__init__(app, name)

        self.object = BrokerProfileParameter(app, "id")
        self.add_parameter(self.object)

        view = BrokerProfileView(app, "view")
        self.add_mode(view)
        self.set_view_mode(view)

        edit = BrokerProfileEdit(app, "edit")
        self.add_mode(edit)
        self.set_edit_mode(edit)

        remove = BrokerProfileRemove(app, "remove")
        self.add_mode(remove)
        self.set_remove_mode(remove)
        
class BrokerProfileView(CuminView):
    def __init__(self, app, name):
        super(BrokerProfileView, self).__init__(app, name)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)
        
        self.tabs.add_tab(self.ProfileConfigTab(app, "config"))
        self.tabs.add_tab(self.ProfileBrokerTab(app, "brokers"))

    class ProfileConfigTab(ConfigPropertySet):
        def get_args(self, session):
            return self.frame.get_args(session)

        def do_get_items(self, session, profile):
            return sorted_by(profile.properties)
        
        def render_title(self, session, profile):
            return "Configuration"

    class ProfileBrokerTab(BrokerSet):
        def __init__(self, app, name):
            super(BrokerProfileView.ProfileBrokerTab, self).__init__(app, name)

        def get_args(self, session):
            return self.frame.get_args(session)

        def render_title(self, session, profile):
            return "Brokers %s" % fmt_count(len(profile.brokers))

        def do_get_items(self, session, profile):
            return profile.brokers

        def render_item_config_href(self, session, broker):
            branch = session.branch()
            frame = self.page.show_broker(branch, broker)
            frame.show_view(branch).show_config(branch)
            return branch.marshal()

        def render_item_config_status(self, session, broker):
            return "0 differences" #XXX

            diffs = 0

            for prop in broker.profile.properties:
                # XXX need to add assoc for this
                for iprop in broker.config_property_items():
                    if iprop.name == prop.name:
                        if iprop.value != prop.value:
                            diffs += 1
                            
            return "%i difference(s)" % diffs

class BrokerProfileForm(CuminForm):
    def __init__(self, app, name):
        super(BrokerProfileForm, self).__init__(app, name)

        self.profile_name = StringInput(app, "name")
        self.add_child(self.profile_name)

    def process_profile(self, session, profile):
        profile.name = self.profile_name.get(session)

        branch = session.branch()
        self.page.show_broker_profile(branch, profile).show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

class BrokerProfileAdd(BrokerProfileForm, Frame):
    def render_title(self, session):
        return "Add Profile"

    def process_cancel(self, session):
        branch = session.branch()
        self.page.main.show(branch)
        self.page.set_redirect_url(session, branch.marshal())
    
    def process_submit(self, session):
        profile = BrokerProfile()
        self.process_profile(session, profile)
    
class BrokerProfileEdit(BrokerProfileForm, Frame):
    def get_args(self, session):
        return self.frame.get_args(session)

    def process_cancel(self, session, profile):
        branch = session.branch()
        self.parent.show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, profile):
        self.process_profile(session, profile)

    def process_display(self, session, profile):
        self.profile_name.set(session, profile.name)

    def render_title(self, session, profile):
        return "Edit Profile '%s'" % profile.name

class BrokerProfileRemove(CuminConfirmForm):
    def get_args(self, session):
        return self.frame.get_args(session)

    def process_cancel(self, session, profile):
        branch = session.branch()
        self.page.show_broker_profile(branch, profile).show_view(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def process_submit(self, session, profile):
        profile.destroySelf()

        branch = session.branch()
        self.page.main.show(branch)
        self.page.set_redirect_url(session, branch.marshal())

    def render_title(self, session, profile):
        return "Remove Broker Profile '%s'" % profile.name

    def render_submit_content(self, session, profile):
        return "Yes, Remove Broker Profile '%s'" % profile.name

    def render_cancel_content(self, session, profile):
        return "No, Cancel"
