from time import sleep
from threading import Thread
from random import sample, random, randint

#from model import *
from mint import *

def fmt(str_, int_):
    return "%s%02i" % (str_, int_)

class DemoData(object):
    def __init__(self, mult=5):
        self.mult = mult

    def load(self):
        groups = dict()

        for name in ("West Coast", "East Coast", "Marketing", "Sales"):
            group = BrokerGroup(name=name)
            groups[name] = group

        CollectorRegistration(name="main", collectorId="main")

        #profiles = list()

        #for name in ("4S Xserve", "2S ProLiant"):
        #    profile = BrokerProfile(name=name)
        #    profiles.append(profile)

        #    self.load_props(profile)

        #clusters = list()

        #for cluster_count in range(3):
        #    cluster = BrokerCluster(name=fmt("cluster", cluster_count))
        #    clusters.append(cluster)

    def load_props(self, obj):
        prop = ConfigProperty()
        prop.name = "max_threads"
        prop.value = 1000
        prop.broker_value = 1000
        prop.type = "integer"
        obj.add_config_property(prop)

        prop = ConfigProperty()
        prop.name = "max_memory"
        prop.value = 1000000
        prop.broker_value = 2000000
        prop.type = "integer"
        obj.add_config_property(prop)

        prop = ConfigProperty()
        prop.name = "ssl_enabled"
        prop.value = True
        prop.broker_value = True
        prop.type = "boolean"
        obj.add_config_property(prop)

    def start_updates(self):
        raise Exception()

        thread = UpdateThread(self.model)
        thread.start()

class UpdateThread(Thread):
    def __init__(self, model):
        super(UpdateThread, self).__init__()

        self.model = model
        self.setDaemon(True)

    def frob(self, mobject):
        mobject.lock()
        try:
            if isinstance(mobject, DummyModelObject):
                for measure in mobject.measurements:
                    self.frob_measure(measure)

            if hasattr(mobject, "errors"):
                self.frob_errors(mobject.errors)

            if hasattr(mobject, "warnings"):
                self.frob_errors(mobject.warnings)
        finally:
            mobject.unlock()

    def frob_measure(self, measure):
        if measure.type == "int":
            if random() < 0.5:
                if random() < 0.66:
                    value = measure.get_value()
                    measure.add_value(value + randint(4, 12))
                else:
                    value = measure.get_value() - randint(3, 9)
                    measure.add_value(value > 0 and value or 0)
            else:
                measure.add_value(measure.get_value())
                
    def frob_errors(self, errors):
        if random() < 0.005:
            errors.append(object())

        if random() < 0.02 and len(errors):
            del errors[0]


    def run(self):
        while True:
            sleep(1)

            for broker in self.model.get_brokers():
                self.frob(broker)

                for vhost in broker.virtual_host_items():
                    self.frob(vhost)
                    
                    for exchange in vhost.exchange_items():
                        self.frob(exchange)
                        
                        for binding in exchange.binding_items():
                            self.frob(binding)

                        for producer in exchange.producer_items():
                            self.frob(producer)

                    for queue in vhost.queue_items():
                        self.frob(queue)

                        for consumer in queue.consumer_items():
                            self.frob(consumer)

                    for conn in vhost.conn_items():
                        self.frob(conn)

                        for session in conn.session_items():
                            self.frob(session)
                        
if __name__ == "__main__":
    import sys

    try:
        connuri = sys.argv[1]
        conn = connectionForURI(connuri)
        sqlhub.processConnection = conn
    except IndexError:
        print "Usage: demo.py DATABASE-URI"
        sys.exit(1)

    data = DemoData()
    data.load()

    #sys.stdout.write("<?xml version=\"1.0\"?><model>")

    #for broker in model.get_brokers():
    #    broker.write_xml(sys.stdout)

    #sys.stdout.write("</model>")
