from wooly import *
from wooly.widgets import *
from wooly.resources import *
from time import sleep

from broker import *
from brokergroup import *
from brokerprofile import *
from brokercluster import *
from pool import *
from job import *
from system import *
from user import *
from action import *
from widgets import *
from util import *

# We have a wooly.widgets.Link and a mint.Link; we want the former
from wooly.widgets import Link
from wooly import Session

strings = StringCatalog(__file__)

class MainPage(CuminPage, ModeSet):
    def __init__(self, app, name):
        super(MainPage, self).__init__(app, name)

        self.main = MainFrame(app, "main")
        self.add_mode(self.main)
        self.set_default_frame(self.main)

    def render_title(self, session):
        return "MRG Management"

class MainFrame(CuminFrame):
    def __init__(self, app, name):
        super(MainFrame, self).__init__(app, name)

        self.__frame_tmpl = Template(self, "frame_html")

        self.__tabs = MainFrameTabs(app, "tabs")
        self.add_child(self.__tabs)

        self.view = MainView(app, "view")
        self.add_mode(self.view)
        self.set_view_mode(self.view)

        self.broker = BrokerFrame(app, "broker")
        self.add_mode(self.broker)

        self.broker_group = BrokerGroupFrame(app, "group")
        self.add_mode(self.broker_group)

        self.system = SystemFrame(app, "system")
        self.add_mode(self.system)

        self.pool = PoolFrame(app, "pool")
        self.add_mode(self.pool)

        self.sticky_pool = Parameter(app, "last_pool")
        self.add_parameter(self.sticky_pool)

        self.change_password = ChangePasswordForm(app, "password")
        self.add_mode(self.change_password)

        self.brokers_add = BrokerSetAdd(app, "brokersadd")
        self.add_mode(self.brokers_add)

        self.brokers_remove = BrokerSetRemove(app, "brokersremove")
        self.add_mode(self.brokers_remove)

        self.jobs_hold = JobSetHold(app, "jobshold")
        self.add_mode(self.jobs_hold)

        self.jobs_release = JobSetRelease(app, "jobsrelease")
        self.add_mode(self.jobs_release)

        self.jobs_remove = JobSetRemove(app, "jobsremove")
        self.add_mode(self.jobs_remove)

        action = self.app.model.broker_group.remove_set
        item = BrokerGroupParameter(app, "item")
        self.broker_groups_remove = CuminSetActionForm \
            (app, "groupsremove", action, item)
        self.add_mode(self.broker_groups_remove)

    def render_title(self, session):
        return "Main"

    def render_user_name(self, session):
        if hasattr(session, "user_session"):
            return session.user_session.subject.name

    def render_logout_href(self, session):
        page = self.app.login_page

        lsess = Session(page)

        page.logout.set(lsess, True)
        page.origin.set(lsess, session.marshal())

        return lsess.marshal()

    def render_frames(self, session):
        writer = Writer()

        for frame in self.page.get_frames(session):
            self.__frame_tmpl.render(writer, session, frame)

        return writer.to_string()

    def render_frame_href(self, session, frame):
        args = frame.get_args(session)
        return frame.render_href(session, *args)

    def render_frame_title(self, session, frame):
        args = frame.get_args(session)
        return frame.render_title(session, *args)

    def render_pending_count(self, session):
        return self.app.model.count_invocations("pending")

    def render_completed_count(self, session):
        return self.app.model.count_invocations("OK")

    def render_failed_count(self, session):
        pcount = self.app.model.count_invocations("pending")
        ccount = self.app.model.count_invocations("OK")
        return len(self.app.model.invocations) - pcount - ccount

    def show_grid_tab(self, session):
        self.__tabs.set_grid_tab(session)
        self.view.set_grid_mode(session)

    def show_system_tab(self, session):
        self.__tabs.set_system_tab(session)
        self.view.set_systems_mode(session)

    def set_messaging_tab(self, session):
        self.__tabs.set_messaging_tab(session)
        self.view.set_messaging_mode(session)

    def set_last_pool(self, session, pool):
        self.sticky_pool.set(session, pool)

    def get_last_pool(self, session):
        return self.sticky_pool.get(session)

class MainFrameTabs(LinkSet):
    def __init__(self, app, name):
        super(MainFrameTabs, self).__init__(app, name)

        self.selection = Parameter(app, "sel")
        self.selection.default = "htab"
        self.add_parameter(self.selection)

        self.add_link(self.HomeTab(app, "htab"))
        self.add_link(self.MessagingTab(app, "mtab"))
        self.add_link(self.GridTab(app, "gtab"))
        self.add_link(self.SystemsTab(app, "stab"))

    def render_class(self, session):
        return "tabs"

    def set_grid_tab(self, session):
        self.selection.set(session, "gtab")

    def set_system_tab(self, session):
        self.selection.set(session, "stab")

    def set_messaging_tab(self, session):
        self.selection.set(session, "mtab")

    class Tab(Link):
        def render_class(self, session):
            return (self.parent.selection.get(session) == self.name) \
                and "selected" or "not-selected"

    class HomeTab(Tab):
        def render_content(self, session):
            return "Home"

        def edit_session(self, session):
            self.parent.selection.set(session, self.name)
            self.page.main.view.home.show(session)

    class MessagingTab(Tab):
        def render_content(self, session):
            return "Messaging"

        def edit_session(self, session):
            self.parent.selection.set(session, self.name)
            self.page.main.view.messaging.show(session)

    class GridTab(Tab):
        def render_content(self, session):
            return "Grid"

        def edit_session(self, session):
            self.parent.selection.set(session, self.name)
            self.page.main.view.grid.show(session)

            last_pool = self.page.main.get_last_pool(session)
            if last_pool:
                pool = model.Pool.get(last_pool)
            else:
                pool = self.app.model.get_main_pool()

            if pool:
                self.page.main.pool.show_object(session, pool)

    class SystemsTab(Tab):
        def render_content(self, session):
            return "Systems"

        def edit_session(self, session):
            self.parent.selection.set(session, self.name)
            self.page.main.view.systems.show(session)

class MainView(ModeSet):
    def __init__(self, app, name):
        super(MainView, self).__init__(app, name)

        self.home = HomeView(app, "home")
        self.add_mode(self.home)

        self.messaging = MessagingView(app, "msg")
        self.add_mode(self.messaging)

        self.grid = GridView(app, "grd")
        self.add_mode(self.grid)

        self.systems = SystemsView(app, "sys")
        self.add_mode(self.systems)

    def set_messaging_mode(self, session):
        self.set_selected_mode(session, self.messaging)

    def set_grid_mode(self, session):
        self.set_selected_mode(session, self.grid)

    def set_systems_mode(self, session):
        self.set_selected_mode(session, self.systems)

class HomeView(TabbedModeSet):
    def __init__(self, app, name):
        super(HomeView, self).__init__(app, name)

        heading = self.Heading(app, "heading")
        self.add_child(heading)

        self.add_tab(self.OverviewTab(app, "over"))
        self.add_tab(self.MyGridJobs(app, "jobs"))
        self.add_tab(self.AccountTab(app, "acct"))

    def render_change_password_href(self, session):
        branch = session.branch()
        self.frame.change_password.show(branch)
        return branch.marshal()

    class Heading(CuminHeading):
        def render_title(self, session):
            return "MRG Management"

        def render_icon_href(self, session):
            return "resource?name=shadowman-36.png"

    class OverviewTab(Widget):
        def __init__(self, app, name):
            super(HomeView.OverviewTab, self).__init__(app, name)

            queues = TopQueueSet(app, "queues")
            self.add_child(queues)

            jobs = TopJobSet(app, "jobs")
            self.add_child(jobs)

            systems = TopSystemSet(app, "systems")
            self.add_child(systems)

        def render_title(self, session):
            return "Overview"

        def render_add_qmf_source_href(self, session):
            branch = session.branch()
            self.page.main.brokers_add.show(branch)
            return branch.marshal()

    class AccountTab(ActionSet):
        def render_title(self, session):
            return "Your Account"

        def render_change_password_href(self, session):
            branch = session.branch()
            self.frame.change_password.show(branch)
            return branch.marshal()

    class MyGridJobs(JobTab):
        def render_title(self, session):
            return "Your Grid Jobs %s" % fmt_count(self.get_item_count(session))

        def render_sql_where(self, session, *args):
            if hasattr(session, "user_session"):
                user = session.user_session.subject.name
            else:
                user = ""
            elems = list()
            elems.append("b.name like '%s%s'" % (user, "%"))
            elems.append(self.get_phase_sql(session))
            return "where %s" % " and ".join(elems)

        def get_sql_values(self, session, *args):
            pass

        def get_visible_columns(self, session):
            return self.get_request_visible_columns(session, ["custom_group", "scheduler"])

        def render_user(self, session, *args):
            if hasattr(session, "user_session"):
                return session.user_session.subject.name

class MessagingView(TabbedModeSet):
    def __init__(self, app, name):
        super(MessagingView, self).__init__(app, name)

        heading = self.Heading(app, "heading")
        self.add_child(heading)

        self.add_tab(BrokerBrowser(app, "brokers"))
        self.add_tab(BrokerGroupSet(app, "groups"))
        #self.add_tab(BrokerProfileSet(app, "profiles"))
        #self.add_tab(BrokerClusterSet(app, "clusters"))

    def render_data_url(self, session):
        return "model.xml"

    class Heading(CuminHeading):
        def render_title(self, session):
            return "Messaging"

        def render_icon_href(self, session):
            return "resource?name=broker-36.png"

class GridView(TabbedModeSet):
    def __init__(self, app, name):
        super(GridView, self).__init__(app, name)

        heading = self.Heading(app, "heading")
        self.add_child(heading)

        self.add_tab(PoolSet(app, "pools"))

    def render_data_url(self, session):
        return "model.xml"

    class Heading(CuminHeading):
        def render_title(self, session):
            return "Grid"

        def render_icon_href(self, session):
            return "resource?name=cluster-36.png"

class SystemsView(TabbedModeSet):
    def __init__(self, app, name):
        super(SystemsView, self).__init__(app, name)

        heading = self.Heading(app, "heading")
        self.add_child(heading)

        self.add_tab(SystemSet(app, "systems"))

    def render_data_url(self, session):
        return "model.xml"

    class Heading(CuminHeading):
        def render_title(self, session):
            return "Systems"

        def render_icon_href(self, session):
            return "resource?name=system-36.png"
