import model
from wooly import *
from mint import *

class CuminObjectParameter(Parameter):
    def __init__(self, app, name, cumin_class):
        self.cumin_class = cumin_class

    def do_unmarshal(self, string):
        return self.cumin_class.mint_class.get(int(string))

    def do_marshal(self, object):
        return str(object.id)

class CuminClassParameter(Parameter):
    def do_unmarshal(self, string):
        return getattr(self.app.model, string, None)

    def do_marshal(self, cls):
        return cls.cumin_name

class BrokerClusterParameter(Parameter):
    def do_unmarshal(self, string):
        return BrokerCluster.get(int(string))

    def do_marshal(self, cluster):
        return str(cluster.id)

class BrokerGroupParameter(Parameter):
    def do_unmarshal(self, string):
        if string == "__none__":
            object = None
        else:
            try:
                object = BrokerGroup.get(int(string))
            except SQLObjectNotFound:
                object = None

        return object

    def do_marshal(self, group):
        return str(group.id)

class BrokerRegistrationParameter(Parameter):
    def do_unmarshal(self, string):
        return BrokerRegistration.get(int(string))

    def do_marshal(self, broker):
        return str(broker.id)

class BrokerProfileParameter(Parameter):
    def do_unmarshal(self, string):
        return BrokerProfile.get(int(string))

    def do_marshal(self, profile):
        return str(profile.id)

class ConnectionParameter(Parameter):
    def do_unmarshal(self, string):
        return ClientConnection.get(int(string))

    def do_marshal(self, conn):
        return str(conn.id)

class ConfigPropertyParameter(Parameter):
    def do_unmarshal(self, string):
        return ConfigProperty.get(int(string))

    def do_marshal(self, prop):
        return str(prop.id)

class ExchangeParameter(Parameter):
    def do_unmarshal(self, string):
        return Exchange.get(int(string))

    def do_marshal(self, exchange):
        return str(exchange.id)

class JobParameter(Parameter):
    def do_unmarshal(self, string):
        return Job.get(int(string))

    def do_marshal(self, job):
        return str(job.id)

class JobGroup(object):
    def __init__(self, id):
        self.id = id

    def get_id(self):
        return self.id

class JobGroupParameter(Parameter):
    def do_unmarshal(self, string):
        return JobGroup(string)

    def do_marshal(self, job_group):
        return str(job_group.id)

class Limit(object):
    def __init__(self, id):
        self.id = id
        
    def get_id(self):
        return self.id
    
class LimitParameter(Parameter):
    def do_unmarshal(self, string):
        return Limit(string)
    
    def do_marshal(self, limit):
        return limit.id

class PeerParameter(Parameter):
    def do_unmarshal(self, string):
        return Link.get(int(string))

    def do_marshal(self, peer):
        return str(peer.id)

class PoolParameter(Parameter):
    def do_unmarshal(self, string):
        for coll in Collector.selectBy(Pool=string):
            break

        return model.Pool(coll)

    def do_marshal(self, pool):
        return str(pool.id)

class QueueParameter(Parameter):
    def do_unmarshal(self, string):
        return Queue.get(int(string))

    def do_marshal(self, queue):
        return str(queue.id)

class SchedulerParameter(Parameter):
    def do_unmarshal(self, string):
        return Scheduler.get(int(string))

    def do_marshal(self, sched):
        return str(sched.id)

class SessionParameter(Parameter):
    def do_unmarshal(self, string):
        return Session.get(int(string))

    def do_marshal(self, session):
        return str(session.id)

class SubmitterParameter(Parameter):
    def do_unmarshal(self, string):
        return Submitter.get(int(string))

    def do_marshal(self, sub):
        return str(sub.id)

class CollectorParameter(Parameter):
    def do_unmarshal(self, string):
        return Collector.get(int(string))

    def do_marshal(self, coll):
        return str(coll.id)

class NegotiatorParameter(Parameter):
    def do_unmarshal(self, string):
        return Negotiator.get(int(string))

    def do_marshal(self, neg):
        return str(neg.id)

class SystemParameter(Parameter):
    def do_unmarshal(self, string):
        return Sysimage.get(int(string))

    def do_marshal(self, session):
        return str(session.id)

class VirtualHostParameter(Parameter):
    def do_unmarshal(self, string):
        return Vhost.get(int(string))

    def do_marshal(self, vhost):
        return str(vhost.id)
