import sys, qpid
from time import sleep
from random import random, sample, randint

class Exchange(object):
    def __init__(self, name, type="direct"):
        self.name = name
        self.type = type

    def declare(self, session):
        session.psession.exchange_declare(exchange=self.name, type=self.type)

    def exists(self, session):
        return False #XXX figure out how to do this

class Queue(object):
    def __init__(self, name):
        self.name = name

    def declare(self, session):
        session.psession.queue_declare(queue=self.name)

    def exists(self, session):
        return False #XXX figure out how to do this

    def bind(self, session, exchange, key=None):
        if key is None:
            key = self.name

        session.psession.queue_bind(exchange=exchange.name,
                                    queue=self.name,
                                    routing_key=key)

class Subscription(object):
    def __init__(self, queue):
        self.name = queue.name # XXX bad?
        self.queue = queue
        self.client_queue = None

    def init(self, session):
        session.psession.message_subscribe(queue=self.queue.name,
                                           destination=self.name)

        session.psession.message_flow(self.name, 0, 0xFFFFFFFF)
        session.psession.message_flow(self.name, 1, 0xFFFFFFFF)

        self.client_queue = session.client.pconn.queue(self.name)

    def get(self, session):
        if self.client_queue is None:
            raise Exception()

        m = Message()
        m.content = self.client_queue.get(timeout=10).content

        return m

class Message(object):
    def __init__(self, payload=None):
        self.content = qpid.content.Content()
        self.content["content_type"] = "text/plain"

        if payload is not None:
            self.set_payload(payload)

    def set_payload(self, payload):
        self.content.body = payload
        
    def send(self, session, dest, key=None):
        if dest.__class__ is Queue:
            self.content["routing_key"] = dest.name
            session.psession.message_transfer(destination="",
                                              content=self.content)
        elif dest.__class__ is Exchange:
            if routing_key is None:
                raise Exception("Routing key not set")

            session.psession.message_transfer(destination=dest.name,
                                              content=self.content)
        else:
            raise Exception("Unknown destination object")

    def __str__(self):
        return self.content.body

class Connection(object):
    def __init__(self, host, port, spec_path):
        self.host = host
        self.port = port
        self.spec = qpid.spec.load(spec_path)

        self.pconn = None

    def open(self):
        assert self.pconn is None

        sock = qpid.util.connect(self.host, self.port)
        self.pconn = qpid.connection.Connection(sock, self.spec)
        self.pconn.start()

    def close(self):
        assert self.pconn

        self.pconn.close()

class Session(object):
    def __init__(self, conn, name):
        self.conn = conn
        self.name = name
        self.psession = conn.pconn.session(name)

    def open(self):
        pass

    def close(self):
        self.psession.close()
