import logging

from wooly import *
from wooly.widgets import *
from wooly.forms import *
from wooly.resources import *
from wooly.tables import *

from stat import *
from widgets import *
from parameters import *
from formats import *
from util import *
from job import *

strings = StringCatalog(__file__)
log = logging.getLogger("cumin.submitter")

class SubmitterSet(CuminTable):
    def __init__(self, app, name):
        super(SubmitterSet, self).__init__(app, name)

        col = self.NameColumn(app, "name")
        self.add_column(col)

        self.set_default_column(col)

    def render_title(self, session):
        return "Submitters %s" % fmt_count(Submitter.select().count())

    class NameColumn(SqlTableColumn):
        def render_title(self, session, data):
            return "Name"

        def render_content(self, session, data):
            sub = Identifiable(data["id"])
            href = self.page.main.pool.submitter.get_href(session, sub)
            return fmt_link(href, data["name"])

class SubmitterFrame(CuminFrame):
    def __init__(self, app, name):
        super(SubmitterFrame, self).__init__(app, name)

        self.object = SubmitterParameter(app, "id")
        self.add_parameter(self.object)

        view = SubmitterView(app, "view")
        self.add_mode(view)
        self.set_view_mode(view)

class SubmitterView(CuminView):
    def __init__(self, app, name):
        super(SubmitterView, self).__init__(app, name)

        status = SubmitterStatus(app, "status")
        self.add_child(status)

        self.__tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.__tabs)

        stats = SubmitterStats(app, "stats")
        self.__tabs.add_tab(stats)

        jobs = SubmitterJobSet(app, "jobs")
        self.__tabs.add_tab(jobs)
        
        details = CuminDetails(app, "details")
        self.__tabs.add_tab(details)

    def render_script(self, session, sched):
        data = "model.xml?class=submitter;id=%i" % sched.id
        return "wooly.setIntervalUpdate('%s', updateSubmitter, 3000)" % data

class SubmitterStatus(CuminStatus):
    def render_title(self, session, submitter):
        return "Submitter Status"

class SubmitterStats(Widget):
    def __init__(self, app, name):
        super(SubmitterStats, self).__init__(app, name)

        stats = StatSet(app, "general", "general")
        self.add_child(stats)

        chart = self.JobsChart(app, "jobs")
        self.add_child(chart)

    def render_title(self, session):
        return "Statistics"

    class JobsChart(StatValueChart):
        def __init__(self, app, name):
            super(SubmitterStats.JobsChart, self).__init__ \
                (app, name)

            self.stats = ("RunningJobs", "IdleJobs", "HeldJobs")

        def render_title(self, session, sched):
            return "Jobs"

class SubmitterJobSet(JobTab):
    def __init__(self, app, name):
        super(SubmitterJobSet, self).__init__(app, name)
        
    def get_visible_columns(self, session):
        return self.get_request_visible_columns(session, ["custom_group", "scheduler"])

    def render_sql_where(self, session, submitter):
        phase_sql = self.get_phase_sql(session)
        submitter_sql = "j.submitter_id = %i" % submitter.id
        return "where %s" % " and ".join([phase_sql, submitter_sql])

    def render_title(self, session, submitter):
        submitter_sql = "submitter_id = %i" % submitter.id
        return "Jobs %s" % fmt_count(Job.select(submitter_sql).count())

