import sys
from datetime import datetime, timedelta
from logging import getLogger
from time import mktime, time, sleep
from random import randint
from xml.sax.saxutils import escape as xml_escape
from crypt import crypt
from random import sample

def short_id():
    return "%08x" % randint(0, sys.maxint)

def sorted_by(seq, attr="name"):
    return sorted(seq, cmp, lambda x: getattr(x, attr))

def ess(num, ending="s"):
    return num != 1 and ending or ""

def nvl(expr1, expr2):
    if expr1 == None:
        return expr2
    else:
        return expr1

def calc_rate(curr, prev, csecs, psecs):
    if None not in (curr, prev, csecs, psecs):
        secs = csecs - psecs
        if secs > 0:
            return (curr - prev) / float(secs)

def secs(dt):
    if dt is not None:
        return mktime(dt.timetuple())

def parse_broker_addr(string):
    addr = string.split(":")

    if len(addr) > 1:
        host, port = addr[0], int(addr[1])
    else:
        host, port = addr[0], 5672

    return host, port

def is_active(obj):
    delTime = obj._get_qmfDeleteTime()
    if not delTime:
        updateTime = obj.statsCurr.qmfUpdateTime
        delta = timedelta(minutes=10)
        # mirroring logic in widgets/PhaseSwitch.get_sql_constraint
        if not updateTime or (updateTime > datetime.now() - delta):
            return True

class Identifiable(object):
    def __init__(self, id=None):
        self.id = id

def convertStr(s):
    """Convert string to either int or float."""
    try:
        ret = int(s)
    except ValueError:
        try:
            ret = float(s)
        except ValueError:
            ret = None
    return ret

def wait(predicate, timeout=30):
    start = time()

    while True:
        if predicate():
            return True

        if time() - start > timeout:
            return False

        sleep(1)

password_chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

def crypt_password(password, salt=None):
    if not salt:
        salt = "".join(sample(password_chars, 2))

    return crypt(password, salt)

def parse(text, begin_delim="{", end_delim="}"):
    strings = list()

    start = 0
    end = text.find(begin_delim)

    while True:
        if (end == -1):
            strings.append(text[start:])
            break

        strings.append(text[start:end])

        ccurly = text.find(end_delim, end + 1)

        if ccurly == -1:
            start = end
            end = -1
        else:
            ocurly = text.find(begin_delim, end + 1)

            if ocurly == -1:
                start = end
                end = ccurly + 1
            elif ocurly < ccurly:
                start = end
                end = ocurly
            else:
                strings.append(begin_delim + text[end + 1:ccurly] + end_delim)
                    
                start = ccurly + 1
                end = ocurly

    return strings
