from wooly import *
from wooly.widgets import *

from queue import *
from exchange import *
from client import *
from widgets import *
from parameters import *
from formats import *
from util import *

strings = StringCatalog(__file__)

class VirtualHostSet(ItemSet):
    def render_title(self, session, *args):
        return "Functional Hosts %s" % \
               fmt_count(len(model.get_virtual_hosts()))

    def do_get_items(self, session, *args):
        return sorted_by(model.get_virtual_hosts())

    def render_item_link(self, session, vhost):
        branch = session.branch()
        self.page.show_virtual_host(branch, vhost).show_view(branch)
        return fmt_olink(branch, vhost)

class VirtualHostFrame(CuminFrame):
    def __init__(self, app, name):
        super(VirtualHostFrame, self).__init__(app, name)

        self.object = VirtualHostParameter(app, "id")
        self.add_parameter(self.object)

        self.view = VirtualHostView(app, "view")
        self.add_mode(self.view)
        self.set_view_mode(self.view)

        self.queue = QueueFrame(app, "queue")
        self.add_mode(self.queue)

        self.__exchange = ExchangeFrame(app, "exchange")
        self.add_mode(self.exchange)

        self.conn = ConnectionFrame(app, "conn")
        self.add_mode(self.conn)

class VirtualHostView(CuminView):
    def __init__(self, app, name):
        super(VirtualHostView, self).__init__(app, name)

        self.tabs = TabbedModeSet(app, "tabs")
        self.add_child(self.tabs)

        self.tabs.add_tab(QueueSet(app, "queues"))
        self.tabs.add_tab(ExchangeSet(app, "exchanges"))
        self.tabs.add_tab(ConnectionSet(app, "conns"))

    def render_broker_link(self, session, vhost):
        broker = vhost.get_broker()

        if broker:
            branch = session.branch()
            self.page.show_broker(branch, broker).show_view(branch)
            return fmt_olink(branch, broker)
        else:
            return fmt_none()

    def render_cluster_link(self, session, vhost):
        cluster = vhost.get_broker_cluster()

        if cluster:
            branch = session.branch()
            self.page.show_broker_cluster(branch, cluster).show_view(branch)
            return fmt_olink(branch, cluster)
        else:
            return fmt_none()
