import logging
import mint
import time
from threading import Thread
from mint.schema import *
from sql import *
from util import *

log = logging.getLogger("mint.dbexpire")

class DBExpireThread(MintDaemonThread):
  def __init__(self, model, frequency, threshold, keepCurrStats=False):
    super(DBExpireThread, self).__init__(model)

    self.frequency = frequency
    self.threshold = threshold
    self.keepCurrStats = keepCurrStats

    self.ops = []
    for cls in mint.schema.statsClasses:
      self.ops.append(SqlExpire(eval(cls), self.keepCurrStats))
    self.ops.append(SqlExpire(Job, self.keepCurrStats))

    self.attrs = dict()
    self.attrs["threshold"] = self.threshold
    
    frequency_out, frequency_unit = self.__convertTimeUnits(frequency)
    threshold_out, threshold_unit = self.__convertTimeUnits(threshold)
    log.debug("Expiring database records older than %d %s, every %d %s" \
                % (threshold_out, threshold_unit, frequency_out, frequency_unit))

  def run(self):
    while True:
      if self.stopRequested:
        break
      up = mint.update.DBExpireUpdate(self.model)
      self.model.updateThread.enqueue(up)
      time.sleep(self.frequency)

  def __convertTimeUnits(self, t):
    if t / (24*3600) >= 1:
      t_out = t / (24*3600)
      t_unit = "days"
    elif t / 3600 >= 1:
      t_out = t / 3600
      t_unit = "hours"
    elif t / 60 >= 1:
      t_out = t / 60
      t_unit = "minutes"
    else:
      t_out = t
      t_unit = "seconds"
    return (t_out, t_unit)
