import mint
from sqlobject import *
from datetime import datetime
from qmf.console import ObjectId

class Pool(SQLObject):
  class sqlmeta:
    lazyUpdate = True
  sourceId = StringCol(length=1000, default=None, unique=True)


class Slot(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SlotStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SlotStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  Pool = StringCol(length=1000, default=None)
  System = StringCol(length=1000, default=None)
  AccountingGroup = StringCol(length=1000, default=None)
  Arch = StringCol(length=1000, default=None)
  CheckpointPlatform = StringCol(length=1000, default=None)
  ClientMachine = StringCol(length=1000, default=None)
  ConcurrencyLimits = StringCol(length=1000, default=None)
  Cpus = BigIntCol(default=None)
  CurrentRank = FloatCol(default=None)
  Disk = BigIntCol(default=None)
  FileSystemDomain = StringCol(length=1000, default=None)
  GlobalJobId = StringCol(length=1000, default=None)
  ImageSize = BigIntCol(default=None)
  IsValidCheckpointPlatform = StringCol(length=4000, default=None)
  JobId = StringCol(length=1000, default=None)
  JobStart = TimestampCol(default=None)
  KFlops = BigIntCol(default=None)
  Machine = StringCol(length=1000, default=None)
  MaxJobRetirementTime = StringCol(length=4000, default=None)
  Memory = BigIntCol(default=None)
  Mips = BigIntCol(default=None)
  MyAddress = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  OpSys = StringCol(length=1000, default=None)
  PreemptingConcurrencyLimits = StringCol(length=1000, default=None)
  PreemptingOwner = StringCol(length=1000, default=None)
  PreemptingUser = StringCol(length=1000, default=None)
  PreemptingRank = FloatCol(default=None)
  RemoteOwner = StringCol(length=1000, default=None)
  RemoteUser = StringCol(length=1000, default=None)
  Requirements = StringCol(length=4000, default=None)
  PublicNetworkIpAddr = StringCol(length=1000, default=None)
  Rank = StringCol(length=4000, default=None)
  SlotID = BigIntCol(default=None)
  Start = StringCol(length=4000, default=None)
  StarterAbilityList = StringCol(length=4000, default=None)
  TotalClaimRunTime = BigIntCol(default=None)
  TotalClaimSuspendTime = BigIntCol(default=None)
  TotalCpus = BigIntCol(default=None)
  TotalDisk = BigIntCol(default=None)
  TotalJobRunTime = BigIntCol(default=None)
  TotalJobSuspendTime = BigIntCol(default=None)
  TotalMemory = BigIntCol(default=None)
  TotalSlots = BigIntCol(default=None)
  TotalVirtualMemory = BigIntCol(default=None)
  UidDomain = StringCol(length=1000, default=None)
  VirtualMemory = BigIntCol(default=None)
  WindowsBuildNumber = BigIntCol(default=None)
  WindowsMajorVersion = BigIntCol(default=None)
  WindowsMinorVersion = BigIntCol(default=None)

  CondorPlatform = StringCol(length=1000, default=None)
  CondorVersion = StringCol(length=1000, default=None)
  DaemonStartTime = TimestampCol(default=None)


class SlotStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  slot = ForeignKey('Slot', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  Activity = StringCol(length=1000, default=None)
  ClockDay = BigIntCol(default=None)
  ClockMin = BigIntCol(default=None)
  CondorLoadAvg = FloatCol(default=None)
  ConsoleIdle = BigIntCol(default=None)
  EnteredCurrentActivity = TimestampCol(default=None)
  EnteredCurrentState = TimestampCol(default=None)
  KeyboardIdle = BigIntCol(default=None)
  LastBenchmark = TimestampCol(default=None)
  LastFetchWorkCompleted = TimestampCol(default=None)
  LastFetchWorkSpawned = TimestampCol(default=None)
  LastPeriodicCheckpoint = TimestampCol(default=None)
  LoadAvg = FloatCol(default=None)
  MyCurrentTime = TimestampCol(default=None)
  NextFetchWorkDelay = IntCol(default=None)
  State = StringCol(length=1000, default=None)
  TimeToLive = BigIntCol(default=None)
  TotalCondorLoadAvg = FloatCol(default=None)
  TotalLoadAvg = FloatCol(default=None)
  TotalTimeBackfillBusy = BigIntCol(default=None)
  TotalTimeBackfillIdle = BigIntCol(default=None)
  TotalTimeBackfillKilling = BigIntCol(default=None)
  TotalTimeClaimedBusy = BigIntCol(default=None)
  TotalTimeClaimedIdle = BigIntCol(default=None)
  TotalTimeClaimedRetiring = BigIntCol(default=None)
  TotalTimeClaimedSuspended = BigIntCol(default=None)
  TotalTimeMatchedIdle = BigIntCol(default=None)
  TotalTimeOwnerIdle = BigIntCol(default=None)
  TotalTimePreemptingKilling = BigIntCol(default=None)
  TotalTimePreemptingVacating = BigIntCol(default=None)
  TotalTimeUnclaimedBenchmarking = BigIntCol(default=None)
  TotalTimeUnclaimedIdle = BigIntCol(default=None)

  MonitorSelfAge = BigIntCol(default=None)
  MonitorSelfCPUUsage = FloatCol(default=None)
  MonitorSelfImageSize = FloatCol(default=None)
  MonitorSelfRegisteredSocketCount = BigIntCol(default=None)
  MonitorSelfResidentSetSize = BigIntCol(default=None)
  MonitorSelfTime = TimestampCol(default=None)




class Job(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('JobStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('JobStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  scheduler = ForeignKey('Scheduler', cascade='null', default=None)
  submitter = ForeignKey('Submitter', cascade='null', default=None)
  AccountingGroup = StringCol(length=1000, default=None)
  Args = StringCol(length=4000, default=None)
  ClusterId = BigIntCol(default=None)
  Cmd = StringCol(length=4000, default=None)
  ConcurrencyLimits = StringCol(length=4000, default=None)
  CustomGroup = StringCol(length=1000, default=None)
  CustomId = StringCol(length=1000, default=None)
  CustomPriority = BigIntCol(default=None)
  GlobalJobId = StringCol(length=1000, default=None)
  InRsv = StringCol(length=4000, default=None)
  Iwd = StringCol(length=4000, default=None)
  JobStatus = BigIntCol(default=None)
  Note = StringCol(length=4000, default=None)
  Out = StringCol(length=4000, default=None)
  Owner = StringCol(length=1000, default=None)
  GridUser = StringCol(length=1000, default=None)
  ProcId = BigIntCol(default=None)
  QDate = TimestampCol(default=None)
  JobUniverse = BigIntCol(default=None)
  Title = StringCol(length=1000, default=None)
  UserLog = StringCol(length=4000, default=None)
  HoldReason = StringCol(length=4000, default=None)
  DAGNodeName = StringCol(length=1000, default=None)
  DAGParentNodeNames = StringCol(length=4000, default=None)
  DAGManJobId = BigIntCol(default=None)
  Ad = PickleCol(default=None)


  def GetAd(self, model, callback, JobAd):
    actualArgs = list()
    if JobAd is not None:
        actualArgs.append(JobAd)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "GetAd",
                     callback, args=actualArgs)

  def SetAttribute(self, model, callback, Name, Value):
    actualArgs = list()
    if Name is not None:
        actualArgs.append(Name)
    if Value is not None:
        actualArgs.append(Value)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "SetAttribute",
                     callback, args=actualArgs)

  def Hold(self, model, callback, Reason):
    actualArgs = list()
    if Reason is not None:
        actualArgs.append(Reason)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Hold",
                     callback, args=actualArgs)

  def Release(self, model, callback, Reason):
    actualArgs = list()
    if Reason is not None:
        actualArgs.append(Reason)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Release",
                     callback, args=actualArgs)

  def Remove(self, model, callback, Reason):
    actualArgs = list()
    if Reason is not None:
        actualArgs.append(Reason)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Remove",
                     callback, args=actualArgs)

  def Fetch(self, model, callback, File, Start, End, Data):
    actualArgs = list()
    if File is not None:
        actualArgs.append(File)
    if Start is not None:
        actualArgs.append(Start)
    if End is not None:
        actualArgs.append(End)
    if Data is not None:
        actualArgs.append(Data)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Fetch",
                     callback, args=actualArgs)

class JobStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  job = ForeignKey('Job', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Scheduler(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SchedulerStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SchedulerStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  Pool = StringCol(length=1000, default=None)
  System = StringCol(length=1000, default=None)
  JobQueueBirthdate = TimestampCol(default=None)
  MaxJobsRunning = BigIntCol(default=None)
  Machine = StringCol(length=1000, default=None)
  MyAddress = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  PublicNetworkIpAddr = StringCol(length=1000, default=None)

  CondorPlatform = StringCol(length=1000, default=None)
  CondorVersion = StringCol(length=1000, default=None)
  DaemonStartTime = TimestampCol(default=None)


class SchedulerStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  scheduler = ForeignKey('Scheduler', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  NumUsers = BigIntCol(default=None)
  TotalHeldJobs = BigIntCol(default=None)
  TotalIdleJobs = BigIntCol(default=None)
  TotalJobAds = BigIntCol(default=None)
  TotalRemovedJobs = BigIntCol(default=None)
  TotalRunningJobs = BigIntCol(default=None)

  MonitorSelfAge = BigIntCol(default=None)
  MonitorSelfCPUUsage = FloatCol(default=None)
  MonitorSelfImageSize = FloatCol(default=None)
  MonitorSelfRegisteredSocketCount = BigIntCol(default=None)
  MonitorSelfResidentSetSize = BigIntCol(default=None)
  MonitorSelfTime = TimestampCol(default=None)




class Submitter(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SubmitterStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SubmitterStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  scheduler = ForeignKey('Scheduler', cascade='null', default=None)
  JobQueueBirthdate = TimestampCol(default=None)
  Machine = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  ScheddName = StringCol(length=1000, default=None)


class SubmitterStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  submitter = ForeignKey('Submitter', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  HeldJobs = BigIntCol(default=None)
  IdleJobs = BigIntCol(default=None)
  RunningJobs = BigIntCol(default=None)




class Negotiator(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('NegotiatorStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('NegotiatorStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  Pool = StringCol(length=1000, default=None)
  System = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  Machine = StringCol(length=1000, default=None)
  MyAddress = StringCol(length=1000, default=None)

  CondorPlatform = StringCol(length=1000, default=None)
  CondorVersion = StringCol(length=1000, default=None)
  DaemonStartTime = TimestampCol(default=None)


  def GetLimits(self, model, callback, Limits):
    actualArgs = list()
    if Limits is not None:
        actualArgs.append(Limits)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "GetLimits",
                     callback, args=actualArgs)

  def SetLimit(self, model, callback, Name, Max):
    actualArgs = list()
    if Name is not None:
        actualArgs.append(Name)
    if Max is not None:
        actualArgs.append(Max)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "SetLimit",
                     callback, args=actualArgs)

  def Reconfig(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Reconfig",
                     callback, args=actualArgs)

class NegotiatorStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  negotiator = ForeignKey('Negotiator', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo

  MonitorSelfAge = BigIntCol(default=None)
  MonitorSelfCPUUsage = FloatCol(default=None)
  MonitorSelfImageSize = FloatCol(default=None)
  MonitorSelfRegisteredSocketCount = BigIntCol(default=None)
  MonitorSelfResidentSetSize = BigIntCol(default=None)
  MonitorSelfTime = TimestampCol(default=None)




class Collector(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('CollectorStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('CollectorStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  Pool = StringCol(length=1000, default=None)
  System = StringCol(length=1000, default=None)
  CondorPlatform = StringCol(length=1000, default=None)
  CondorVersion = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  PublicNetworkIpAddr = StringCol(length=1000, default=None)


class CollectorStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  collector = ForeignKey('Collector', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Master(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('MasterStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('MasterStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  Pool = StringCol(length=1000, default=None)
  System = StringCol(length=1000, default=None)
  Name = StringCol(length=1000, default=None)
  Machine = StringCol(length=1000, default=None)
  PublicNetworkIpAddr = StringCol(length=1000, default=None)
  MyAddress = StringCol(length=1000, default=None)
  RealUid = IntCol(default=None)

  CondorPlatform = StringCol(length=1000, default=None)
  CondorVersion = StringCol(length=1000, default=None)
  DaemonStartTime = TimestampCol(default=None)


  def Start(self, model, callback, Subsystem):
    actualArgs = list()
    if Subsystem is not None:
        actualArgs.append(Subsystem)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Start",
                     callback, args=actualArgs)

  def Stop(self, model, callback, Subsystem):
    actualArgs = list()
    if Subsystem is not None:
        actualArgs.append(Subsystem)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "Stop",
                     callback, args=actualArgs)

class MasterStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  master = ForeignKey('Master', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo

  MonitorSelfAge = BigIntCol(default=None)
  MonitorSelfCPUUsage = FloatCol(default=None)
  MonitorSelfImageSize = FloatCol(default=None)
  MonitorSelfRegisteredSocketCount = BigIntCol(default=None)
  MonitorSelfResidentSetSize = BigIntCol(default=None)
  MonitorSelfTime = TimestampCol(default=None)




class Acl(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('AclStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('AclStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  broker = ForeignKey('Broker', cascade='null', default=None)
  policyFile = StringCol(length=1000, default=None)
  enforcingAcl = BoolCol(default=None)
  transferAcl = BoolCol(default=None)
  lastAclLoad = TimestampCol(default=None)


  def reloadACLFile(self, model, callback):
    """Reload the ACL file"""
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "reloadACLFile",
                     callback, args=actualArgs)

class AclStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  acl = ForeignKey('Acl', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  aclDenyCount = BigIntCol(default=None)




class Cluster(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('ClusterStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('ClusterStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  broker = ForeignKey('Broker', cascade='null', default=None)
  clusterName = StringCol(length=1000, default=None)
  clusterID = StringCol(length=1000, default=None)
  publishedURL = StringCol(length=1000, default=None)
  clusterSize = IntCol(default=None)
  status = StringCol(length=1000, default=None)
  members = StringCol(length=4000, default=None)


  def stopClusterNode(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "stopClusterNode",
                     callback, args=actualArgs)

  def stopFullCluster(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "stopFullCluster",
                     callback, args=actualArgs)

class ClusterStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  cluster = ForeignKey('Cluster', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Store(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('StoreStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('StoreStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  broker = ForeignKey('Broker', cascade='null', default=None)
  location = StringCol(length=1000, default=None)
  defaultInitialFileCount = IntCol(default=None)
  defaultDataFileSize = BigIntCol(default=None)
  tplIsInitialized = BoolCol(default=None)
  tplDirectory = StringCol(length=1000, default=None)
  tplWritePageSize = BigIntCol(default=None)
  tplWritePages = BigIntCol(default=None)
  tplInitialFileCount = IntCol(default=None)
  tplDataFileSize = BigIntCol(default=None)
  tplCurrentFileCount = BigIntCol(default=None)


class StoreStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  store = ForeignKey('Store', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  tplTransactionDepth = BigIntCol(default=None)
  tplTransactionDepthLow = BigIntCol(default=None)
  tplTransactionDepthHigh = BigIntCol(default=None)
  tplTxnPrepares = BigIntCol(default=None)
  tplTxnCommits = BigIntCol(default=None)
  tplTxnAborts = BigIntCol(default=None)
  tplOutstandingAIOs = BigIntCol(default=None)
  tplOutstandingAIOsLow = BigIntCol(default=None)
  tplOutstandingAIOsHigh = BigIntCol(default=None)




class Journal(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('JournalStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('JournalStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  queue = ForeignKey('Queue', cascade='null', default=None)
  name = StringCol(length=1000, default=None)
  directory = StringCol(length=1000, default=None)
  baseFileName = StringCol(length=1000, default=None)
  writePageSize = BigIntCol(default=None)
  writePages = BigIntCol(default=None)
  readPageSize = BigIntCol(default=None)
  readPages = BigIntCol(default=None)
  initialFileCount = IntCol(default=None)
  dataFileSize = BigIntCol(default=None)
  currentFileCount = BigIntCol(default=None)


  def expand(self, model, callback, by):
    """Increase number of files allocated for this journal"""
    actualArgs = list()
    if by is not None:
        actualArgs.append(by)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "expand",
                     callback, args=actualArgs)

class JournalStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  journal = ForeignKey('Journal', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  recordDepth = BigIntCol(default=None)
  recordDepthLow = BigIntCol(default=None)
  recordDepthHigh = BigIntCol(default=None)
  enqueues = BigIntCol(default=None)
  dequeues = BigIntCol(default=None)
  txnEnqueues = BigIntCol(default=None)
  txnDequeues = BigIntCol(default=None)
  txnCommits = BigIntCol(default=None)
  txnAborts = BigIntCol(default=None)
  outstandingAIOs = BigIntCol(default=None)
  outstandingAIOsLow = BigIntCol(default=None)
  outstandingAIOsHigh = BigIntCol(default=None)
  freeFileCount = BigIntCol(default=None)
  freeFileCountLow = BigIntCol(default=None)
  freeFileCountHigh = BigIntCol(default=None)
  availableFileCount = BigIntCol(default=None)
  availableFileCountLow = BigIntCol(default=None)
  availableFileCountHigh = BigIntCol(default=None)
  writeWaitFailures = BigIntCol(default=None)
  writeBusyFailures = BigIntCol(default=None)
  readRecordCount = BigIntCol(default=None)
  readBusyFailures = BigIntCol(default=None)
  writePageCacheDepth = BigIntCol(default=None)
  writePageCacheDepthLow = BigIntCol(default=None)
  writePageCacheDepthHigh = BigIntCol(default=None)
  readPageCacheDepth = BigIntCol(default=None)
  readPageCacheDepthLow = BigIntCol(default=None)
  readPageCacheDepthHigh = BigIntCol(default=None)




class System(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SystemStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SystemStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  systemId = BLOBCol(default=None)
  osName = StringCol(length=1000, default=None)
  nodeName = StringCol(length=1000, default=None)
  release = StringCol(length=1000, default=None)
  version = StringCol(length=1000, default=None)
  machine = StringCol(length=1000, default=None)


class SystemStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  system = ForeignKey('System', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Broker(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('BrokerStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('BrokerStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  system = ForeignKey('System', cascade='null', default=None)
  port = BigIntCol(default=None)
  workerThreads = IntCol(default=None)
  maxConns = IntCol(default=None)
  connBacklog = IntCol(default=None)
  stagingThreshold = BigIntCol(default=None)
  mgmtPubInterval = IntCol(default=None)
  version = StringCol(length=1000, default=None)
  dataDir = StringCol(length=1000, default=None)


  def echo(self, model, callback, sequence, body):
    """Request a response to test the path to the management broker"""
    actualArgs = list()
    if sequence is not None:
        actualArgs.append(sequence)
    if body is not None:
        actualArgs.append(body)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "echo",
                     callback, args=actualArgs)

  def connect(self, model, callback, host, port, durable, authMechanism, username, password, transport):
    """Establish a connection to another broker"""
    actualArgs = list()
    if host is not None:
        actualArgs.append(host)
    if port is not None:
        actualArgs.append(port)
    if durable is not None:
        actualArgs.append(durable)
    if authMechanism is not None:
        actualArgs.append(authMechanism)
    if username is not None:
        actualArgs.append(username)
    if password is not None:
        actualArgs.append(password)
    if transport is not None:
        actualArgs.append(transport)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "connect",
                     callback, args=actualArgs)

  def queueMoveMessages(self, model, callback, srcQueue, destQueue, qty):
    """Move messages from one queue to another"""
    actualArgs = list()
    if srcQueue is not None:
        actualArgs.append(srcQueue)
    if destQueue is not None:
        actualArgs.append(destQueue)
    if qty is not None:
        actualArgs.append(qty)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "queueMoveMessages",
                     callback, args=actualArgs)

class BrokerStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  broker = ForeignKey('Broker', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Agent(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('AgentStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('AgentStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  clientConnection = ForeignKey('ClientConnection', cascade='null', default=None)
  label = StringCol(length=1000, default=None)
  broker = ForeignKey('Broker', cascade='null', default=None)
  systemId = BLOBCol(default=None)
  brokerBank = BigIntCol(default=None)
  agentBank = BigIntCol(default=None)


class AgentStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  agent = ForeignKey('Agent', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Vhost(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('VhostStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('VhostStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  broker = ForeignKey('Broker', cascade='null', default=None)
  name = StringCol(length=1000, default=None)
  federationTag = StringCol(length=1000, default=None)


class VhostStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Queue(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('QueueStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('QueueStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  name = StringCol(length=1000, default=None)
  durable = BoolCol(default=None)
  autoDelete = BoolCol(default=None)
  exclusive = BoolCol(default=None)
  arguments = PickleCol(default=None)


  def purge(self, model, callback, request):
    """Discard all or some messages on a queue"""
    actualArgs = list()
    if request is not None:
        actualArgs.append(request)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "purge",
                     callback, args=actualArgs)

class QueueStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  queue = ForeignKey('Queue', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  msgTotalEnqueues = BigIntCol(default=None)
  msgTotalDequeues = BigIntCol(default=None)
  msgTxnEnqueues = BigIntCol(default=None)
  msgTxnDequeues = BigIntCol(default=None)
  msgPersistEnqueues = BigIntCol(default=None)
  msgPersistDequeues = BigIntCol(default=None)
  msgDepth = BigIntCol(default=None)
  byteDepth = BigIntCol(default=None)
  byteTotalEnqueues = BigIntCol(default=None)
  byteTotalDequeues = BigIntCol(default=None)
  byteTxnEnqueues = BigIntCol(default=None)
  byteTxnDequeues = BigIntCol(default=None)
  bytePersistEnqueues = BigIntCol(default=None)
  bytePersistDequeues = BigIntCol(default=None)
  consumerCount = BigIntCol(default=None)
  consumerCountLow = BigIntCol(default=None)
  consumerCountHigh = BigIntCol(default=None)
  bindingCount = BigIntCol(default=None)
  bindingCountLow = BigIntCol(default=None)
  bindingCountHigh = BigIntCol(default=None)
  unackedMessages = BigIntCol(default=None)
  unackedMessagesLow = BigIntCol(default=None)
  unackedMessagesHigh = BigIntCol(default=None)
  messageLatencyMin = BigIntCol(default=None)
  messageLatencyMax = BigIntCol(default=None)
  messageLatencyAverage = BigIntCol(default=None)
  messageLatencySamples = BigIntCol(default=None)




class Exchange(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('ExchangeStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('ExchangeStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  name = StringCol(length=1000, default=None)
  type = StringCol(length=1000, default=None)
  durable = BoolCol(default=None)
  arguments = PickleCol(default=None)


class ExchangeStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  exchange = ForeignKey('Exchange', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  producerCount = BigIntCol(default=None)
  producerCountLow = BigIntCol(default=None)
  producerCountHigh = BigIntCol(default=None)
  bindingCount = BigIntCol(default=None)
  bindingCountLow = BigIntCol(default=None)
  bindingCountHigh = BigIntCol(default=None)
  msgReceives = BigIntCol(default=None)
  msgDrops = BigIntCol(default=None)
  msgRoutes = BigIntCol(default=None)
  byteReceives = BigIntCol(default=None)
  byteDrops = BigIntCol(default=None)
  byteRoutes = BigIntCol(default=None)




class Binding(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('BindingStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('BindingStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  exchange = ForeignKey('Exchange', cascade='null', default=None)
  queue = ForeignKey('Queue', cascade='null', default=None)
  bindingKey = StringCol(length=1000, default=None)
  arguments = PickleCol(default=None)
  origin = StringCol(length=1000, default=None)


class BindingStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  binding = ForeignKey('Binding', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  msgMatched = BigIntCol(default=None)




class ClientConnection(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('ClientConnectionStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('ClientConnectionStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  address = StringCol(length=1000, default=None)
  incoming = BoolCol(default=None)
  SystemConnection = BoolCol(default=None)
  federationLink = BoolCol(default=None)
  authIdentity = StringCol(length=1000, default=None)


  def close(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "close",
                     callback, args=actualArgs)

class ClientConnectionStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  clientConnection = ForeignKey('ClientConnection', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  closing = BoolCol(default=None)
  framesFromClient = BigIntCol(default=None)
  framesToClient = BigIntCol(default=None)
  bytesFromClient = BigIntCol(default=None)
  bytesToClient = BigIntCol(default=None)




class Link(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('LinkStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('LinkStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  host = StringCol(length=1000, default=None)
  port = BigIntCol(default=None)
  transport = StringCol(length=1000, default=None)
  durable = BoolCol(default=None)


  def close(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "close",
                     callback, args=actualArgs)

  def bridge(self, model, callback, durable, src, dest, key, tag, excludes, srcIsQueue, srcIsLocal, dynamic):
    """Bridge messages over the link"""
    actualArgs = list()
    if durable is not None:
        actualArgs.append(durable)
    if src is not None:
        actualArgs.append(src)
    if dest is not None:
        actualArgs.append(dest)
    if key is not None:
        actualArgs.append(key)
    if tag is not None:
        actualArgs.append(tag)
    if excludes is not None:
        actualArgs.append(excludes)
    if srcIsQueue is not None:
        actualArgs.append(srcIsQueue)
    if srcIsLocal is not None:
        actualArgs.append(srcIsLocal)
    if dynamic is not None:
        actualArgs.append(dynamic)
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "bridge",
                     callback, args=actualArgs)

class LinkStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  link = ForeignKey('Link', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  state = StringCol(length=1000, default=None)
  lastError = StringCol(length=1000, default=None)




class Bridge(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('BridgeStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('BridgeStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  link = ForeignKey('Link', cascade='null', default=None)
  channelId = IntCol(default=None)
  durable = BoolCol(default=None)
  src = StringCol(length=1000, default=None)
  dest = StringCol(length=1000, default=None)
  key = StringCol(length=1000, default=None)
  srcIsQueue = BoolCol(default=None)
  srcIsLocal = BoolCol(default=None)
  tag = StringCol(length=1000, default=None)
  excludes = StringCol(length=1000, default=None)
  dynamic = BoolCol(default=None)


  def close(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "close",
                     callback, args=actualArgs)

class BridgeStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  bridge = ForeignKey('Bridge', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo




class Session(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SessionStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SessionStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  vhost = ForeignKey('Vhost', cascade='null', default=None)
  name = StringCol(length=1000, default=None)
  channelId = IntCol(default=None)
  clientConnection = ForeignKey('ClientConnection', cascade='null', default=None)
  detachedLifespan = BigIntCol(default=None)
  attached = BoolCol(default=None)
  expireTime = TimestampCol(default=None)


  def solicitAck(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "solicitAck",
                     callback, args=actualArgs)

  def detach(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "detach",
                     callback, args=actualArgs)

  def resetLifespan(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "resetLifespan",
                     callback, args=actualArgs)

  def close(self, model, callback):
    actualArgs = list()
    originalId = ObjectId(None, self.qmfScopeId, self.qmfObjectId)
    model.callMethod(self.qmfBrokerId, originalId, self.qmfClassKey, "close",
                     callback, args=actualArgs)

class SessionStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  session = ForeignKey('Session', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  framesOutstanding = BigIntCol(default=None)
  TxnStarts = BigIntCol(default=None)
  TxnCommits = BigIntCol(default=None)
  TxnRejects = BigIntCol(default=None)
  TxnCount = BigIntCol(default=None)




class Sysimage(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfBrokerId = StringCol(length=1000, default=None)
  qmfScopeId = BigIntCol(default=None)
  qmfObjectId = BigIntCol(default=None)
  qmfIdsUnique = DatabaseIndex(qmfBrokerId, qmfScopeId, qmfObjectId, unique=True)
  qmfClassKey = StringCol(length=1000, default=None)
  qmfUpdateTime = TimestampCol(default=None)
  qmfCreateTime = TimestampCol(default=None)
  qmfDeleteTime = TimestampCol(default=None)
  statsCurr = ForeignKey('SysimageStats', cascade='null', default=None)
  statsCurrIndex = DatabaseIndex(statsCurr)
  statsPrev = ForeignKey('SysimageStats', cascade='null', default=None)
  statsPrevIndex = DatabaseIndex(statsPrev)
  classInfos = dict() # brokerId => classInfo
  uuid = BLOBCol(default=None)
  osName = StringCol(length=1000, default=None)
  nodeName = StringCol(length=1000, default=None)
  release = StringCol(length=1000, default=None)
  version = StringCol(length=1000, default=None)
  machine = StringCol(length=1000, default=None)
  distro = StringCol(length=1000, default=None)
  memTotal = BigIntCol(default=None)
  swapTotal = BigIntCol(default=None)


class SysimageStats(SQLObject):
  class sqlmeta:
    lazyUpdate = True

  qmfUpdateTime = TimestampCol(default=None)
  sysimage = ForeignKey('Sysimage', cascade='null', default=None)
  classInfos = dict() # brokerId => classInfo
  memFree = BigIntCol(default=None)
  swapFree = BigIntCol(default=None)
  loadAverage1Min = FloatCol(default=None)
  loadAverage5Min = FloatCol(default=None)
  loadAverage10Min = FloatCol(default=None)
  procTotal = BigIntCol(default=None)
  procRunning = BigIntCol(default=None)




classToSchemaNameMap = dict()
schemaNameToClassMap = dict()
schemaReservedWordsMap = {"in": "inRsv", "In": "InRsv", 
  "connection": "clientConnection", "Connection": "ClientConnection", 
  "connectionRef": "clientConnectionRef", 
  "user": "gridUser", "User": "GridUser", 
  "registeredTo": "broker",
  "sync": "syncRsv"} 

classToSchemaNameMap['Slot'] = 'Slot'
schemaNameToClassMap['Slot'] = Slot

Slot.sqlmeta.addJoin(SQLMultipleJoin('SlotStats', joinMethodName='stats'))

classToSchemaNameMap['Job'] = 'Job'
schemaNameToClassMap['Job'] = Job

Scheduler.sqlmeta.addJoin(SQLMultipleJoin('Job', joinMethodName='jobs'))

Submitter.sqlmeta.addJoin(SQLMultipleJoin('Job', joinMethodName='jobs'))


Job.sqlmeta.addJoin(SQLMultipleJoin('JobStats', joinMethodName='stats'))

classToSchemaNameMap['Scheduler'] = 'Scheduler'
schemaNameToClassMap['Scheduler'] = Scheduler

Scheduler.sqlmeta.addJoin(SQLMultipleJoin('SchedulerStats', joinMethodName='stats'))

classToSchemaNameMap['Submitter'] = 'Submitter'
schemaNameToClassMap['Submitter'] = Submitter

Scheduler.sqlmeta.addJoin(SQLMultipleJoin('Submitter', joinMethodName='submitters'))


Submitter.sqlmeta.addJoin(SQLMultipleJoin('SubmitterStats', joinMethodName='stats'))

classToSchemaNameMap['Negotiator'] = 'Negotiator'
schemaNameToClassMap['Negotiator'] = Negotiator

Negotiator.sqlmeta.addJoin(SQLMultipleJoin('NegotiatorStats', joinMethodName='stats'))

classToSchemaNameMap['Collector'] = 'Collector'
schemaNameToClassMap['Collector'] = Collector

Collector.sqlmeta.addJoin(SQLMultipleJoin('CollectorStats', joinMethodName='stats'))

classToSchemaNameMap['Master'] = 'Master'
schemaNameToClassMap['Master'] = Master

Master.sqlmeta.addJoin(SQLMultipleJoin('MasterStats', joinMethodName='stats'))

classToSchemaNameMap['Acl'] = 'Acl'
schemaNameToClassMap['Acl'] = Acl

Broker.sqlmeta.addJoin(SQLMultipleJoin('Acl', joinMethodName='acls'))


Acl.sqlmeta.addJoin(SQLMultipleJoin('AclStats', joinMethodName='stats'))

classToSchemaNameMap['Cluster'] = 'Cluster'
schemaNameToClassMap['Cluster'] = Cluster

Broker.sqlmeta.addJoin(SQLMultipleJoin('Cluster', joinMethodName='clusters'))


Cluster.sqlmeta.addJoin(SQLMultipleJoin('ClusterStats', joinMethodName='stats'))

classToSchemaNameMap['Store'] = 'Store'
schemaNameToClassMap['Store'] = Store

Broker.sqlmeta.addJoin(SQLMultipleJoin('Store', joinMethodName='stores'))


Store.sqlmeta.addJoin(SQLMultipleJoin('StoreStats', joinMethodName='stats'))

classToSchemaNameMap['Journal'] = 'Journal'
schemaNameToClassMap['Journal'] = Journal

Queue.sqlmeta.addJoin(SQLMultipleJoin('Journal', joinMethodName='journals'))


Journal.sqlmeta.addJoin(SQLMultipleJoin('JournalStats', joinMethodName='stats'))

classToSchemaNameMap['System'] = 'System'
schemaNameToClassMap['System'] = System

System.sqlmeta.addJoin(SQLMultipleJoin('SystemStats', joinMethodName='stats'))

classToSchemaNameMap['Broker'] = 'Broker'
schemaNameToClassMap['Broker'] = Broker

System.sqlmeta.addJoin(SQLMultipleJoin('Broker', joinMethodName='brokers'))


Broker.sqlmeta.addJoin(SQLMultipleJoin('BrokerStats', joinMethodName='stats'))

classToSchemaNameMap['Agent'] = 'Agent'
schemaNameToClassMap['Agent'] = Agent

ClientConnection.sqlmeta.addJoin(SQLMultipleJoin('Agent', joinMethodName='agents'))

Broker.sqlmeta.addJoin(SQLMultipleJoin('Agent', joinMethodName='agents'))


Agent.sqlmeta.addJoin(SQLMultipleJoin('AgentStats', joinMethodName='stats'))

classToSchemaNameMap['Vhost'] = 'Vhost'
schemaNameToClassMap['Vhost'] = Vhost

Broker.sqlmeta.addJoin(SQLMultipleJoin('Vhost', joinMethodName='vhosts'))


Vhost.sqlmeta.addJoin(SQLMultipleJoin('VhostStats', joinMethodName='stats'))

classToSchemaNameMap['Queue'] = 'Queue'
schemaNameToClassMap['Queue'] = Queue

Vhost.sqlmeta.addJoin(SQLMultipleJoin('Queue', joinMethodName='queues'))


Queue.sqlmeta.addJoin(SQLMultipleJoin('QueueStats', joinMethodName='stats'))

classToSchemaNameMap['Exchange'] = 'Exchange'
schemaNameToClassMap['Exchange'] = Exchange

Vhost.sqlmeta.addJoin(SQLMultipleJoin('Exchange', joinMethodName='exchanges'))


Exchange.sqlmeta.addJoin(SQLMultipleJoin('ExchangeStats', joinMethodName='stats'))

classToSchemaNameMap['Binding'] = 'Binding'
schemaNameToClassMap['Binding'] = Binding

Exchange.sqlmeta.addJoin(SQLMultipleJoin('Binding', joinMethodName='bindings'))

Queue.sqlmeta.addJoin(SQLMultipleJoin('Binding', joinMethodName='bindings'))


Binding.sqlmeta.addJoin(SQLMultipleJoin('BindingStats', joinMethodName='stats'))

classToSchemaNameMap['ClientConnection'] = 'ClientConnection'
schemaNameToClassMap['ClientConnection'] = ClientConnection

Vhost.sqlmeta.addJoin(SQLMultipleJoin('ClientConnection', joinMethodName='clientConnections'))


ClientConnection.sqlmeta.addJoin(SQLMultipleJoin('ClientConnectionStats', joinMethodName='stats'))

classToSchemaNameMap['Link'] = 'Link'
schemaNameToClassMap['Link'] = Link

Vhost.sqlmeta.addJoin(SQLMultipleJoin('Link', joinMethodName='links'))


Link.sqlmeta.addJoin(SQLMultipleJoin('LinkStats', joinMethodName='stats'))

classToSchemaNameMap['Bridge'] = 'Bridge'
schemaNameToClassMap['Bridge'] = Bridge

Link.sqlmeta.addJoin(SQLMultipleJoin('Bridge', joinMethodName='bridges'))


Bridge.sqlmeta.addJoin(SQLMultipleJoin('BridgeStats', joinMethodName='stats'))

classToSchemaNameMap['Session'] = 'Session'
schemaNameToClassMap['Session'] = Session

Vhost.sqlmeta.addJoin(SQLMultipleJoin('Session', joinMethodName='sessions'))

ClientConnection.sqlmeta.addJoin(SQLMultipleJoin('Session', joinMethodName='sessions'))


Session.sqlmeta.addJoin(SQLMultipleJoin('SessionStats', joinMethodName='stats'))

classToSchemaNameMap['Sysimage'] = 'Sysimage'
schemaNameToClassMap['Sysimage'] = Sysimage

Sysimage.sqlmeta.addJoin(SQLMultipleJoin('SysimageStats', joinMethodName='stats'))


entityClasses = ['Slot', 'Job', 'Scheduler', 'Submitter', 'Negotiator', 'Collector', 'Master', 'Acl', 'Cluster', 'Store', 'Journal', 'System', 'Broker', 'Agent', 'Vhost', 'Queue', 'Exchange', 'Binding', 'ClientConnection', 'Link', 'Bridge', 'Session', 'Sysimage']

statsClasses = ['SlotStats', 'JobStats', 'SchedulerStats', 'SubmitterStats', 'NegotiatorStats', 'CollectorStats', 'MasterStats', 'AclStats', 'ClusterStats', 'StoreStats', 'JournalStats', 'SystemStats', 'BrokerStats', 'AgentStats', 'VhostStats', 'QueueStats', 'ExchangeStats', 'BindingStats', 'ClientConnectionStats', 'LinkStats', 'BridgeStats', 'SessionStats', 'SysimageStats']
