import sys, os
from random import sample
from time import clock
from xml.parsers.expat import ParserCreate
from traceback import print_exc

from wooly import *

class BenchmarkHarness(object):
    def __init__(self, app, check_xml=False):
        self.app = app
        self.check_xml = check_xml
        self.continue_on_error = False

    def run(self, max=-1):
        visited = set()
        self.app.debug.urls.add("")
        count = 0
        
        while True:
            for url in self.app.debug.urls:
                if url not in visited:
                    visited.add(url)
                    break

            print count, url,

            start = clock()

            session = Session.unmarshal(self.app, url)

            try:
                page = session.page
                page.process(session)

                html = page.render(session)
            except KeyboardInterrupt:
                raise
            except:
                print

                session.debug.print_last_call()

                if self.continue_on_error:
                    print_exc(file=sys.stderr)
                else:
                    raise

            if self.check_xml:
                try:
                    parser = ParserCreate()
                    parser.Parse(html)
                except KeyboardInterrupt:
                    raise
                except:
                    print

                    html = html + "[eof]"

                    lines = html.split(os.linesep)

                    for i, line in enumerate(lines):
                        print "%4i %s" % (i + 1, line)

                    if self.continue_on_error:
                        print_exc(file=sys.stderr)
                    else:
                        raise

            bytes = len(html)
            millis = (clock() - start) * 1000

            print "[%i bytes, %i millis]" % (bytes, millis)

            if count == max:
                break
            
            count += 1

class DevelPage(Page):
    html = """
    <html>
      <head>
        <title>{title}</title>
      </head>
      <body>{rtrace}</body>
    </html>
    """

    def __init__(self, app, name):
        super(DevelPage, self).__init__(app, name)

        self.render_trace = self.RenderTrace(app, "rtrace")
        self.add_child(self.render_trace)

    class RenderTrace(Widget):
        def do_render(self, session):
            writer = Writer()

            writer.write("<ul>")

            if self.app.debug and self.app.debug.sessions:
                call = self.app.debug.sessions[-1].render_stack[0]
                self.render_call(session, call, writer)

            writer.write("</ul>")

            return writer.to_string()

        def render_call(self, session, call, writer):
            writer.write("<li>%s</li>" % str(call.widget))
            writer.write("<ul>")

            for c in call.callees:
                self.render_call(session, c, writer)

            writer.write("</ul>")
