from datetime import datetime

from wooly import Page, Parameter, Writer

class CssPage(Page):
    def __init__(self, app, name):
        super(CssPage, self).__init__(app, name)

        self.__then = datetime.utcnow()
        self.__css = None

    def get_last_modified(self, session):
        return self.__then

    def get_content_type(self, session):
        return "text/css"

    def get_css(self):
        if not self.__css:
            writer = Writer()

            for cls in sorted(self.app.widget_classes):
                strs = cls.get_module_strings()

                if strs:
                    css = strs.get(cls.__name__ + ".css")

                    if css:
                        writer.write(css)
                        writer.write("\r\n") # HTTP newline

            self.__css = writer.to_string()

        return self.__css

    def do_render(self, session, *args):
        return self.get_css()

class JavascriptPage(Page):
    def __init__(self, app, name):
        super(JavascriptPage, self).__init__(app, name)

        self.__then = datetime.utcnow()
        self.__javascript = None

    def get_last_modified(self, session):
        return self.__then

    def get_content_type(self, session):
        return "text/javascript"

    def get_javascript(self):
        if not self.__javascript:
            writer = Writer()

            for cls in sorted(self.app.widget_classes):
                strs = cls.get_module_strings()

                if strs:
                    javascript = strs.get(cls.__name__ + ".javascript")

                    if javascript:
                        writer.write(javascript)
                        writer.write("\r\n")

            self.__javascript = writer.to_string()

        return self.__javascript

    def do_render(self, session, *args):
        return self.get_javascript()

class ResourcePage(Page):
    def __init__(self, app, name):
        super(ResourcePage, self).__init__(app, name)

        self.rname = Parameter(app, "name")
        self.add_parameter(self.rname)

        self.then = datetime.utcnow()

    def get_last_modified(self, session):
        return self.then

    def get_content_type(self, session):
        name = self.rname.get(session)

        if name:
            if name.endswith(".png"):
                type = "image/png"
            elif name.endswith(".jpeg"):
                type = "image/jpeg"
            elif name.endswith(".html"):
                type = "text/html"
            elif name.endswith(".css"):
                type = "text/css"
            elif name.endswith(".js"):
                type = "text/javascript"
            else:
                type = "text/plain"

            return type

    def do_render(self, session, *args):
        name = self.rname.get(session)

        if name:
            resource = self.app.get_resource(name)

            if resource:
                return resource.read()
