CREATE TABLE broker_cluster (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000)
);

CREATE TABLE broker_group (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000) NOT NULL UNIQUE
);

CREATE TABLE broker_group_mapping (
    id SERIAL PRIMARY KEY,
    broker_registration_id INT NOT NULL,
    broker_group_id INT NOT NULL
);
CREATE UNIQUE INDEX broker_group_mapping_unique ON broker_group_mapping (broker_registration_id, broker_group_id);

CREATE TABLE broker_profile (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000)
);

CREATE TABLE broker_registration (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000) NOT NULL UNIQUE,
    url VARCHAR(1000),
    broker_id INT,
    cluster_id INT,
    profile_id INT
);
CREATE UNIQUE INDEX broker_registration_url_unique ON broker_registration (url);

CREATE TABLE collector_registration (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000),
    collector_id VARCHAR(1000)
);

CREATE TABLE mint_info (
    id SERIAL PRIMARY KEY,
    version VARCHAR(1000) NOT NULL
);

CREATE TABLE role (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000) NOT NULL UNIQUE
);

CREATE TABLE subject (
    id SERIAL PRIMARY KEY,
    name VARCHAR(1000) NOT NULL UNIQUE,
    password VARCHAR(1000),
    last_challenged TIMESTAMP,
    last_logged_in TIMESTAMP,
    last_logged_out TIMESTAMP
);

CREATE TABLE subject_role_mapping (
    id SERIAL PRIMARY KEY,
    subject_id INT NOT NULL,
    role_id INT NOT NULL
);
CREATE UNIQUE INDEX subject_role_mapping_unique ON subject_role_mapping (subject_id, role_id);

CREATE TABLE acl (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    broker_id INT,
    policy_file VARCHAR(1000),
    enforcing_acl BOOL,
    transfer_acl BOOL,
    last_acl_load TIMESTAMP
);
CREATE UNIQUE INDEX acl_qmfIdsUnique ON acl (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX acl_statsPrevIndex ON acl (stats_prev_id);
CREATE INDEX acl_statsCurrIndex ON acl (stats_curr_id);

CREATE TABLE acl_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    acl_id INT,
    acl_deny_count BIGINT
);

CREATE TABLE agent (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    client_connection_id INT,
    label VARCHAR(1000),
    broker_id INT,
    system_id BYTEA,
    broker_bank BIGINT,
    agent_bank BIGINT
);
CREATE UNIQUE INDEX agent_qmfIdsUnique ON agent (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX agent_statsPrevIndex ON agent (stats_prev_id);
CREATE INDEX agent_statsCurrIndex ON agent (stats_curr_id);

CREATE TABLE agent_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    agent_id INT
);

CREATE TABLE binding (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    exchange_id INT,
    queue_id INT,
    binding_key VARCHAR(1000),
    arguments BYTEA,
    origin VARCHAR(1000)
);
CREATE UNIQUE INDEX binding_qmfIdsUnique ON binding (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX binding_statsPrevIndex ON binding (stats_prev_id);
CREATE INDEX binding_statsCurrIndex ON binding (stats_curr_id);

CREATE TABLE binding_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    binding_id INT,
    msg_matched BIGINT
);

CREATE TABLE bridge (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    link_id INT,
    channel_id INT,
    durable BOOL,
    src VARCHAR(1000),
    dest VARCHAR(1000),
    key VARCHAR(1000),
    src_is_queue BOOL,
    src_is_local BOOL,
    tag VARCHAR(1000),
    excludes VARCHAR(1000),
    dynamic BOOL
);
CREATE UNIQUE INDEX bridge_qmfIdsUnique ON bridge (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX bridge_statsCurrIndex ON bridge (stats_curr_id);
CREATE INDEX bridge_statsPrevIndex ON bridge (stats_prev_id);

CREATE TABLE bridge_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    bridge_id INT
);

CREATE TABLE broker (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    system_id INT,
    port BIGINT,
    worker_threads INT,
    max_conns INT,
    conn_backlog INT,
    staging_threshold BIGINT,
    mgmt_pub_interval INT,
    version VARCHAR(1000),
    data_dir VARCHAR(1000),
    registration_id INT
);
CREATE UNIQUE INDEX broker_qmfIdsUnique ON broker (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX broker_statsCurrIndex ON broker (stats_curr_id);
CREATE INDEX broker_statsPrevIndex ON broker (stats_prev_id);

CREATE TABLE broker_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    broker_id INT
);

CREATE TABLE client_connection (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    vhost_id INT,
    address VARCHAR(1000),
    incoming BOOL,
    system_connection BOOL,
    federation_link BOOL,
    auth_identity VARCHAR(1000)
);
CREATE UNIQUE INDEX client_connection_qmfIdsUnique ON client_connection (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX client_connection_statsCurrIndex ON client_connection (stats_curr_id);
CREATE INDEX client_connection_statsPrevIndex ON client_connection (stats_prev_id);

CREATE TABLE client_connection_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    client_connection_id INT,
    closing BOOL,
    frames_from_client BIGINT,
    frames_to_client BIGINT,
    bytes_from_client BIGINT,
    bytes_to_client BIGINT
);

CREATE TABLE cluster (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    broker_id INT,
    cluster_name VARCHAR(1000),
    cluster_id VARCHAR(1000),
    published_ur_l VARCHAR(1000),
    cluster_size INT,
    status VARCHAR(1000),
    members VARCHAR(4000)
);
CREATE UNIQUE INDEX cluster_qmfIdsUnique ON cluster (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX cluster_statsCurrIndex ON cluster (stats_curr_id);
CREATE INDEX cluster_statsPrevIndex ON cluster (stats_prev_id);

CREATE TABLE cluster_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    cluster_id INT
);

CREATE TABLE collector (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    pool VARCHAR(1000),
    system VARCHAR(1000),
    condor_platform VARCHAR(1000),
    condor_version VARCHAR(1000),
    name VARCHAR(1000),
    public_network_ip_addr VARCHAR(1000)
);
CREATE UNIQUE INDEX collector_qmfIdsUnique ON collector (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX collector_statsPrevIndex ON collector (stats_prev_id);
CREATE INDEX collector_statsCurrIndex ON collector (stats_curr_id);

CREATE TABLE collector_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    collector_id INT
);

CREATE TABLE exchange (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    vhost_id INT,
    name VARCHAR(1000),
    type VARCHAR(1000),
    durable BOOL,
    arguments BYTEA
);
CREATE UNIQUE INDEX exchange_qmfIdsUnique ON exchange (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX exchange_statsPrevIndex ON exchange (stats_prev_id);
CREATE INDEX exchange_statsCurrIndex ON exchange (stats_curr_id);

CREATE TABLE exchange_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    exchange_id INT,
    producer_count BIGINT,
    producer_count_low BIGINT,
    producer_count_high BIGINT,
    binding_count BIGINT,
    binding_count_low BIGINT,
    binding_count_high BIGINT,
    msg_receives BIGINT,
    msg_drops BIGINT,
    msg_routes BIGINT,
    byte_receives BIGINT,
    byte_drops BIGINT,
    byte_routes BIGINT
);

CREATE TABLE job (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    scheduler_id INT,
    submitter_id INT,
    accounting_group VARCHAR(1000),
    args VARCHAR(4000),
    cluster_id BIGINT,
    cmd VARCHAR(4000),
    concurrency_limits VARCHAR(4000),
    custom_group VARCHAR(1000),
    custom_id VARCHAR(1000),
    custom_priority BIGINT,
    global_job_id VARCHAR(1000),
    in_rsv VARCHAR(4000),
    iwd VARCHAR(4000),
    job_status BIGINT,
    note VARCHAR(4000),
    out VARCHAR(4000),
    owner VARCHAR(1000),
    grid_user VARCHAR(1000),
    proc_id BIGINT,
    q_date TIMESTAMP,
    job_universe BIGINT,
    title VARCHAR(1000),
    user_log VARCHAR(4000),
    hold_reason VARCHAR(4000),
    dag_node_name VARCHAR(1000),
    dag_parent_node_names VARCHAR(4000),
    dag_man_job_id BIGINT,
    ad BYTEA
);
CREATE UNIQUE INDEX job_qmfIdsUnique ON job (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX job_statsPrevIndex ON job (stats_prev_id);
CREATE INDEX job_statsCurrIndex ON job (stats_curr_id);

CREATE TABLE job_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    job_id INT
);

CREATE TABLE journal (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    queue_id INT,
    name VARCHAR(1000),
    directory VARCHAR(1000),
    base_file_name VARCHAR(1000),
    write_page_size BIGINT,
    write_pages BIGINT,
    read_page_size BIGINT,
    read_pages BIGINT,
    initial_file_count INT,
    data_file_size BIGINT,
    current_file_count BIGINT
);
CREATE UNIQUE INDEX journal_qmfIdsUnique ON journal (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX journal_statsPrevIndex ON journal (stats_prev_id);
CREATE INDEX journal_statsCurrIndex ON journal (stats_curr_id);

CREATE TABLE journal_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    journal_id INT,
    record_depth BIGINT,
    record_depth_low BIGINT,
    record_depth_high BIGINT,
    enqueues BIGINT,
    dequeues BIGINT,
    txn_enqueues BIGINT,
    txn_dequeues BIGINT,
    txn_commits BIGINT,
    txn_aborts BIGINT,
    outstanding_ai_os BIGINT,
    outstanding_ai_os_low BIGINT,
    outstanding_ai_os_high BIGINT,
    free_file_count BIGINT,
    free_file_count_low BIGINT,
    free_file_count_high BIGINT,
    available_file_count BIGINT,
    available_file_count_low BIGINT,
    available_file_count_high BIGINT,
    write_wait_failures BIGINT,
    write_busy_failures BIGINT,
    read_record_count BIGINT,
    read_busy_failures BIGINT,
    write_page_cache_depth BIGINT,
    write_page_cache_depth_low BIGINT,
    write_page_cache_depth_high BIGINT,
    read_page_cache_depth BIGINT,
    read_page_cache_depth_low BIGINT,
    read_page_cache_depth_high BIGINT
);

CREATE TABLE link (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    vhost_id INT,
    host VARCHAR(1000),
    port BIGINT,
    transport VARCHAR(1000),
    durable BOOL
);
CREATE UNIQUE INDEX link_qmfIdsUnique ON link (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX link_statsCurrIndex ON link (stats_curr_id);
CREATE INDEX link_statsPrevIndex ON link (stats_prev_id);

CREATE TABLE link_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    link_id INT,
    state VARCHAR(1000),
    last_error VARCHAR(1000)
);

CREATE TABLE master (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    pool VARCHAR(1000),
    system VARCHAR(1000),
    name VARCHAR(1000),
    machine VARCHAR(1000),
    public_network_ip_addr VARCHAR(1000),
    my_address VARCHAR(1000),
    real_uid INT,
    condor_platform VARCHAR(1000),
    condor_version VARCHAR(1000),
    daemon_start_time TIMESTAMP
);
CREATE UNIQUE INDEX master_qmfIdsUnique ON master (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX master_statsCurrIndex ON master (stats_curr_id);
CREATE INDEX master_statsPrevIndex ON master (stats_prev_id);

CREATE TABLE master_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    master_id INT,
    monitor_self_age BIGINT,
    monitor_self_cpu_usage FLOAT,
    monitor_self_image_size FLOAT,
    monitor_self_registered_socket_count BIGINT,
    monitor_self_resident_set_size BIGINT,
    monitor_self_time TIMESTAMP
);

CREATE TABLE negotiator (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    pool VARCHAR(1000),
    system VARCHAR(1000),
    name VARCHAR(1000),
    machine VARCHAR(1000),
    my_address VARCHAR(1000),
    condor_platform VARCHAR(1000),
    condor_version VARCHAR(1000),
    daemon_start_time TIMESTAMP
);
CREATE UNIQUE INDEX negotiator_qmfIdsUnique ON negotiator (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX negotiator_statsPrevIndex ON negotiator (stats_prev_id);
CREATE INDEX negotiator_statsCurrIndex ON negotiator (stats_curr_id);

CREATE TABLE negotiator_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    negotiator_id INT,
    monitor_self_age BIGINT,
    monitor_self_cpu_usage FLOAT,
    monitor_self_image_size FLOAT,
    monitor_self_registered_socket_count BIGINT,
    monitor_self_resident_set_size BIGINT,
    monitor_self_time TIMESTAMP
);

CREATE TABLE pool (
    id SERIAL PRIMARY KEY,
    source_id VARCHAR(1000) UNIQUE
);

CREATE TABLE queue (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    vhost_id INT,
    name VARCHAR(1000),
    durable BOOL,
    auto_delete BOOL,
    exclusive BOOL,
    arguments BYTEA
);
CREATE UNIQUE INDEX queue_qmfIdsUnique ON queue (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX queue_statsCurrIndex ON queue (stats_curr_id);
CREATE INDEX queue_statsPrevIndex ON queue (stats_prev_id);

CREATE TABLE queue_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    queue_id INT,
    msg_total_enqueues BIGINT,
    msg_total_dequeues BIGINT,
    msg_txn_enqueues BIGINT,
    msg_txn_dequeues BIGINT,
    msg_persist_enqueues BIGINT,
    msg_persist_dequeues BIGINT,
    msg_depth BIGINT,
    byte_depth BIGINT,
    byte_total_enqueues BIGINT,
    byte_total_dequeues BIGINT,
    byte_txn_enqueues BIGINT,
    byte_txn_dequeues BIGINT,
    byte_persist_enqueues BIGINT,
    byte_persist_dequeues BIGINT,
    consumer_count BIGINT,
    consumer_count_low BIGINT,
    consumer_count_high BIGINT,
    binding_count BIGINT,
    binding_count_low BIGINT,
    binding_count_high BIGINT,
    unacked_messages BIGINT,
    unacked_messages_low BIGINT,
    unacked_messages_high BIGINT,
    message_latency_min BIGINT,
    message_latency_max BIGINT,
    message_latency_average BIGINT,
    message_latency_samples BIGINT
);

CREATE TABLE scheduler (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    pool VARCHAR(1000),
    system VARCHAR(1000),
    job_queue_birthdate TIMESTAMP,
    max_jobs_running BIGINT,
    machine VARCHAR(1000),
    my_address VARCHAR(1000),
    name VARCHAR(1000),
    public_network_ip_addr VARCHAR(1000),
    condor_platform VARCHAR(1000),
    condor_version VARCHAR(1000),
    daemon_start_time TIMESTAMP
);
CREATE UNIQUE INDEX scheduler_qmfIdsUnique ON scheduler (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX scheduler_statsCurrIndex ON scheduler (stats_curr_id);
CREATE INDEX scheduler_statsPrevIndex ON scheduler (stats_prev_id);

CREATE TABLE scheduler_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    scheduler_id INT,
    num_users BIGINT,
    total_held_jobs BIGINT,
    total_idle_jobs BIGINT,
    total_job_ads BIGINT,
    total_removed_jobs BIGINT,
    total_running_jobs BIGINT,
    monitor_self_age BIGINT,
    monitor_self_cpu_usage FLOAT,
    monitor_self_image_size FLOAT,
    monitor_self_registered_socket_count BIGINT,
    monitor_self_resident_set_size BIGINT,
    monitor_self_time TIMESTAMP
);

CREATE TABLE session (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    vhost_id INT,
    name VARCHAR(1000),
    channel_id INT,
    client_connection_id INT,
    detached_lifespan BIGINT,
    attached BOOL,
    expire_time TIMESTAMP
);
CREATE UNIQUE INDEX session_qmfIdsUnique ON session (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX session_statsCurrIndex ON session (stats_curr_id);
CREATE INDEX session_statsPrevIndex ON session (stats_prev_id);

CREATE TABLE session_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    session_id INT,
    frames_outstanding BIGINT,
    txn_starts BIGINT,
    txn_commits BIGINT,
    txn_rejects BIGINT,
    txn_count BIGINT
);

CREATE TABLE slot (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    pool VARCHAR(1000),
    system VARCHAR(1000),
    accounting_group VARCHAR(1000),
    arch VARCHAR(1000),
    checkpoint_platform VARCHAR(1000),
    client_machine VARCHAR(1000),
    concurrency_limits VARCHAR(1000),
    cpus BIGINT,
    current_rank FLOAT,
    disk BIGINT,
    file_system_domain VARCHAR(1000),
    global_job_id VARCHAR(1000),
    image_size BIGINT,
    is_valid_checkpoint_platform VARCHAR(4000),
    job_id VARCHAR(1000),
    job_start TIMESTAMP,
    k_flops BIGINT,
    machine VARCHAR(1000),
    max_job_retirement_time VARCHAR(4000),
    memory BIGINT,
    mips BIGINT,
    my_address VARCHAR(1000),
    name VARCHAR(1000),
    op_sys VARCHAR(1000),
    preempting_concurrency_limits VARCHAR(1000),
    preempting_owner VARCHAR(1000),
    preempting_user VARCHAR(1000),
    preempting_rank FLOAT,
    remote_owner VARCHAR(1000),
    remote_user VARCHAR(1000),
    requirements VARCHAR(4000),
    public_network_ip_addr VARCHAR(1000),
    rank VARCHAR(4000),
    slot_id BIGINT,
    start VARCHAR(4000),
    starter_ability_list VARCHAR(4000),
    total_claim_run_time BIGINT,
    total_claim_suspend_time BIGINT,
    total_cpus BIGINT,
    total_disk BIGINT,
    total_job_run_time BIGINT,
    total_job_suspend_time BIGINT,
    total_memory BIGINT,
    total_slots BIGINT,
    total_virtual_memory BIGINT,
    uid_domain VARCHAR(1000),
    virtual_memory BIGINT,
    windows_build_number BIGINT,
    windows_major_version BIGINT,
    windows_minor_version BIGINT,
    condor_platform VARCHAR(1000),
    condor_version VARCHAR(1000),
    daemon_start_time TIMESTAMP
);
CREATE UNIQUE INDEX slot_qmfIdsUnique ON slot (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX slot_statsCurrIndex ON slot (stats_curr_id);
CREATE INDEX slot_statsPrevIndex ON slot (stats_prev_id);

CREATE TABLE slot_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    slot_id INT,
    activity VARCHAR(1000),
    clock_day BIGINT,
    clock_min BIGINT,
    condor_load_avg FLOAT,
    console_idle BIGINT,
    entered_current_activity TIMESTAMP,
    entered_current_state TIMESTAMP,
    keyboard_idle BIGINT,
    last_benchmark TIMESTAMP,
    last_fetch_work_completed TIMESTAMP,
    last_fetch_work_spawned TIMESTAMP,
    last_periodic_checkpoint TIMESTAMP,
    load_avg FLOAT,
    my_current_time TIMESTAMP,
    next_fetch_work_delay INT,
    state VARCHAR(1000),
    time_to_live BIGINT,
    total_condor_load_avg FLOAT,
    total_load_avg FLOAT,
    total_time_backfill_busy BIGINT,
    total_time_backfill_idle BIGINT,
    total_time_backfill_killing BIGINT,
    total_time_claimed_busy BIGINT,
    total_time_claimed_idle BIGINT,
    total_time_claimed_retiring BIGINT,
    total_time_claimed_suspended BIGINT,
    total_time_matched_idle BIGINT,
    total_time_owner_idle BIGINT,
    total_time_preempting_killing BIGINT,
    total_time_preempting_vacating BIGINT,
    total_time_unclaimed_benchmarking BIGINT,
    total_time_unclaimed_idle BIGINT,
    monitor_self_age BIGINT,
    monitor_self_cpu_usage FLOAT,
    monitor_self_image_size FLOAT,
    monitor_self_registered_socket_count BIGINT,
    monitor_self_resident_set_size BIGINT,
    monitor_self_time TIMESTAMP
);

CREATE TABLE store (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    broker_id INT,
    location VARCHAR(1000),
    default_initial_file_count INT,
    default_data_file_size BIGINT,
    tpl_is_initialized BOOL,
    tpl_directory VARCHAR(1000),
    tpl_write_page_size BIGINT,
    tpl_write_pages BIGINT,
    tpl_initial_file_count INT,
    tpl_data_file_size BIGINT,
    tpl_current_file_count BIGINT
);
CREATE UNIQUE INDEX store_qmfIdsUnique ON store (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX store_statsCurrIndex ON store (stats_curr_id);
CREATE INDEX store_statsPrevIndex ON store (stats_prev_id);

CREATE TABLE store_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    store_id INT,
    tpl_transaction_depth BIGINT,
    tpl_transaction_depth_low BIGINT,
    tpl_transaction_depth_high BIGINT,
    tpl_txn_prepares BIGINT,
    tpl_txn_commits BIGINT,
    tpl_txn_aborts BIGINT,
    tpl_outstanding_ai_os BIGINT,
    tpl_outstanding_ai_os_low BIGINT,
    tpl_outstanding_ai_os_high BIGINT
);

CREATE TABLE submitter (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    scheduler_id INT,
    job_queue_birthdate TIMESTAMP,
    machine VARCHAR(1000),
    name VARCHAR(1000),
    schedd_name VARCHAR(1000)
);
CREATE UNIQUE INDEX submitter_qmfIdsUnique ON submitter (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX submitter_statsPrevIndex ON submitter (stats_prev_id);
CREATE INDEX submitter_statsCurrIndex ON submitter (stats_curr_id);

CREATE TABLE submitter_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    submitter_id INT,
    held_jobs BIGINT,
    idle_jobs BIGINT,
    running_jobs BIGINT
);

CREATE TABLE sysimage (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    uuid BYTEA,
    os_name VARCHAR(1000),
    node_name VARCHAR(1000),
    release VARCHAR(1000),
    version VARCHAR(1000),
    machine VARCHAR(1000),
    distro VARCHAR(1000),
    mem_total BIGINT,
    swap_total BIGINT
);
CREATE UNIQUE INDEX sysimage_qmfIdsUnique ON sysimage (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX sysimage_statsPrevIndex ON sysimage (stats_prev_id);
CREATE INDEX sysimage_statsCurrIndex ON sysimage (stats_curr_id);

CREATE TABLE sysimage_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    sysimage_id INT,
    mem_free BIGINT,
    swap_free BIGINT,
    load_average1_min FLOAT,
    load_average5_min FLOAT,
    load_average10_min FLOAT,
    proc_total BIGINT,
    proc_running BIGINT
);

CREATE TABLE system (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    system_id BYTEA,
    os_name VARCHAR(1000),
    node_name VARCHAR(1000),
    release VARCHAR(1000),
    version VARCHAR(1000),
    machine VARCHAR(1000)
);
CREATE UNIQUE INDEX system_qmfIdsUnique ON system (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX system_statsPrevIndex ON system (stats_prev_id);
CREATE INDEX system_statsCurrIndex ON system (stats_curr_id);

CREATE TABLE system_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    system_id INT
);

CREATE TABLE vhost (
    id SERIAL PRIMARY KEY,
    qmf_broker_id VARCHAR(1000),
    qmf_scope_id BIGINT,
    qmf_object_id BIGINT,
    qmf_class_key VARCHAR(1000),
    qmf_update_time TIMESTAMP,
    qmf_create_time TIMESTAMP,
    qmf_delete_time TIMESTAMP,
    stats_curr_id INT,
    stats_prev_id INT,
    broker_id INT,
    name VARCHAR(1000),
    federation_tag VARCHAR(1000)
);
CREATE UNIQUE INDEX vhost_qmfIdsUnique ON vhost (qmf_broker_id, qmf_scope_id, qmf_object_id);
CREATE INDEX vhost_statsPrevIndex ON vhost (stats_prev_id);
CREATE INDEX vhost_statsCurrIndex ON vhost (stats_curr_id);

CREATE TABLE vhost_stats (
    id SERIAL PRIMARY KEY,
    qmf_update_time TIMESTAMP,
    vhost_id INT
);

ALTER TABLE broker_group_mapping ADD CONSTRAINT broker_registration_id_exists FOREIGN KEY (broker_registration_id) REFERENCES broker_registration (id) ON DELETE CASCADE;

ALTER TABLE broker_group_mapping ADD CONSTRAINT broker_group_id_exists FOREIGN KEY (broker_group_id) REFERENCES broker_group (id) ON DELETE CASCADE;

ALTER TABLE broker_registration ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE broker_registration ADD CONSTRAINT cluster_id_exists FOREIGN KEY (cluster_id) REFERENCES broker_cluster (id) ON DELETE SET NULL;

ALTER TABLE broker_registration ADD CONSTRAINT profile_id_exists FOREIGN KEY (profile_id) REFERENCES broker_profile (id) ON DELETE SET NULL;

ALTER TABLE subject_role_mapping ADD CONSTRAINT subject_id_exists FOREIGN KEY (subject_id) REFERENCES subject (id) ON DELETE CASCADE;

ALTER TABLE subject_role_mapping ADD CONSTRAINT role_id_exists FOREIGN KEY (role_id) REFERENCES role (id) ON DELETE CASCADE;

ALTER TABLE acl ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES acl_stats (id) ON DELETE SET NULL;

ALTER TABLE acl ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES acl_stats (id) ON DELETE SET NULL;

ALTER TABLE acl ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE acl_stats ADD CONSTRAINT acl_id_exists FOREIGN KEY (acl_id) REFERENCES acl (id) ON DELETE SET NULL;

ALTER TABLE agent ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES agent_stats (id) ON DELETE SET NULL;

ALTER TABLE agent ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES agent_stats (id) ON DELETE SET NULL;

ALTER TABLE agent ADD CONSTRAINT client_connection_id_exists FOREIGN KEY (client_connection_id) REFERENCES client_connection (id) ON DELETE SET NULL;

ALTER TABLE agent ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE agent_stats ADD CONSTRAINT agent_id_exists FOREIGN KEY (agent_id) REFERENCES agent (id) ON DELETE SET NULL;

ALTER TABLE binding ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES binding_stats (id) ON DELETE SET NULL;

ALTER TABLE binding ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES binding_stats (id) ON DELETE SET NULL;

ALTER TABLE binding ADD CONSTRAINT exchange_id_exists FOREIGN KEY (exchange_id) REFERENCES exchange (id) ON DELETE SET NULL;

ALTER TABLE binding ADD CONSTRAINT queue_id_exists FOREIGN KEY (queue_id) REFERENCES queue (id) ON DELETE SET NULL;

ALTER TABLE binding_stats ADD CONSTRAINT binding_id_exists FOREIGN KEY (binding_id) REFERENCES binding (id) ON DELETE SET NULL;

ALTER TABLE bridge ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES bridge_stats (id) ON DELETE SET NULL;

ALTER TABLE bridge ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES bridge_stats (id) ON DELETE SET NULL;

ALTER TABLE bridge ADD CONSTRAINT link_id_exists FOREIGN KEY (link_id) REFERENCES link (id) ON DELETE SET NULL;

ALTER TABLE bridge_stats ADD CONSTRAINT bridge_id_exists FOREIGN KEY (bridge_id) REFERENCES bridge (id) ON DELETE SET NULL;

ALTER TABLE broker ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES broker_stats (id) ON DELETE SET NULL;

ALTER TABLE broker ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES broker_stats (id) ON DELETE SET NULL;

ALTER TABLE broker ADD CONSTRAINT system_id_exists FOREIGN KEY (system_id) REFERENCES system (id) ON DELETE SET NULL;

ALTER TABLE broker ADD CONSTRAINT registration_id_exists FOREIGN KEY (registration_id) REFERENCES broker_registration (id) ON DELETE SET NULL;

ALTER TABLE broker_stats ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE client_connection ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES client_connection_stats (id) ON DELETE SET NULL;

ALTER TABLE client_connection ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES client_connection_stats (id) ON DELETE SET NULL;

ALTER TABLE client_connection ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

ALTER TABLE client_connection_stats ADD CONSTRAINT client_connection_id_exists FOREIGN KEY (client_connection_id) REFERENCES client_connection (id) ON DELETE SET NULL;

ALTER TABLE cluster ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES cluster_stats (id) ON DELETE SET NULL;

ALTER TABLE cluster ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES cluster_stats (id) ON DELETE SET NULL;

ALTER TABLE cluster ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE cluster_stats ADD CONSTRAINT cluster_id_exists FOREIGN KEY (cluster_id) REFERENCES cluster (id) ON DELETE SET NULL;

ALTER TABLE collector ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES collector_stats (id) ON DELETE SET NULL;

ALTER TABLE collector ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES collector_stats (id) ON DELETE SET NULL;

ALTER TABLE collector_stats ADD CONSTRAINT collector_id_exists FOREIGN KEY (collector_id) REFERENCES collector (id) ON DELETE SET NULL;

ALTER TABLE exchange ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES exchange_stats (id) ON DELETE SET NULL;

ALTER TABLE exchange ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES exchange_stats (id) ON DELETE SET NULL;

ALTER TABLE exchange ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

ALTER TABLE exchange_stats ADD CONSTRAINT exchange_id_exists FOREIGN KEY (exchange_id) REFERENCES exchange (id) ON DELETE SET NULL;

ALTER TABLE job ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES job_stats (id) ON DELETE SET NULL;

ALTER TABLE job ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES job_stats (id) ON DELETE SET NULL;

ALTER TABLE job ADD CONSTRAINT scheduler_id_exists FOREIGN KEY (scheduler_id) REFERENCES scheduler (id) ON DELETE SET NULL;

ALTER TABLE job ADD CONSTRAINT submitter_id_exists FOREIGN KEY (submitter_id) REFERENCES submitter (id) ON DELETE SET NULL;

ALTER TABLE job_stats ADD CONSTRAINT job_id_exists FOREIGN KEY (job_id) REFERENCES job (id) ON DELETE SET NULL;

ALTER TABLE journal ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES journal_stats (id) ON DELETE SET NULL;

ALTER TABLE journal ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES journal_stats (id) ON DELETE SET NULL;

ALTER TABLE journal ADD CONSTRAINT queue_id_exists FOREIGN KEY (queue_id) REFERENCES queue (id) ON DELETE SET NULL;

ALTER TABLE journal_stats ADD CONSTRAINT journal_id_exists FOREIGN KEY (journal_id) REFERENCES journal (id) ON DELETE SET NULL;

ALTER TABLE link ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES link_stats (id) ON DELETE SET NULL;

ALTER TABLE link ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES link_stats (id) ON DELETE SET NULL;

ALTER TABLE link ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

ALTER TABLE link_stats ADD CONSTRAINT link_id_exists FOREIGN KEY (link_id) REFERENCES link (id) ON DELETE SET NULL;

ALTER TABLE master ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES master_stats (id) ON DELETE SET NULL;

ALTER TABLE master ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES master_stats (id) ON DELETE SET NULL;

ALTER TABLE master_stats ADD CONSTRAINT master_id_exists FOREIGN KEY (master_id) REFERENCES master (id) ON DELETE SET NULL;

ALTER TABLE negotiator ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES negotiator_stats (id) ON DELETE SET NULL;

ALTER TABLE negotiator ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES negotiator_stats (id) ON DELETE SET NULL;

ALTER TABLE negotiator_stats ADD CONSTRAINT negotiator_id_exists FOREIGN KEY (negotiator_id) REFERENCES negotiator (id) ON DELETE SET NULL;

ALTER TABLE queue ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES queue_stats (id) ON DELETE SET NULL;

ALTER TABLE queue ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES queue_stats (id) ON DELETE SET NULL;

ALTER TABLE queue ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

ALTER TABLE queue_stats ADD CONSTRAINT queue_id_exists FOREIGN KEY (queue_id) REFERENCES queue (id) ON DELETE SET NULL;

ALTER TABLE scheduler ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES scheduler_stats (id) ON DELETE SET NULL;

ALTER TABLE scheduler ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES scheduler_stats (id) ON DELETE SET NULL;

ALTER TABLE scheduler_stats ADD CONSTRAINT scheduler_id_exists FOREIGN KEY (scheduler_id) REFERENCES scheduler (id) ON DELETE SET NULL;

ALTER TABLE session ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES session_stats (id) ON DELETE SET NULL;

ALTER TABLE session ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES session_stats (id) ON DELETE SET NULL;

ALTER TABLE session ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

ALTER TABLE session ADD CONSTRAINT client_connection_id_exists FOREIGN KEY (client_connection_id) REFERENCES client_connection (id) ON DELETE SET NULL;

ALTER TABLE session_stats ADD CONSTRAINT session_id_exists FOREIGN KEY (session_id) REFERENCES session (id) ON DELETE SET NULL;

ALTER TABLE slot ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES slot_stats (id) ON DELETE SET NULL;

ALTER TABLE slot ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES slot_stats (id) ON DELETE SET NULL;

ALTER TABLE slot_stats ADD CONSTRAINT slot_id_exists FOREIGN KEY (slot_id) REFERENCES slot (id) ON DELETE SET NULL;

ALTER TABLE store ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES store_stats (id) ON DELETE SET NULL;

ALTER TABLE store ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES store_stats (id) ON DELETE SET NULL;

ALTER TABLE store ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE store_stats ADD CONSTRAINT store_id_exists FOREIGN KEY (store_id) REFERENCES store (id) ON DELETE SET NULL;

ALTER TABLE submitter ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES submitter_stats (id) ON DELETE SET NULL;

ALTER TABLE submitter ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES submitter_stats (id) ON DELETE SET NULL;

ALTER TABLE submitter ADD CONSTRAINT scheduler_id_exists FOREIGN KEY (scheduler_id) REFERENCES scheduler (id) ON DELETE SET NULL;

ALTER TABLE submitter_stats ADD CONSTRAINT submitter_id_exists FOREIGN KEY (submitter_id) REFERENCES submitter (id) ON DELETE SET NULL;

ALTER TABLE sysimage ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES sysimage_stats (id) ON DELETE SET NULL;

ALTER TABLE sysimage ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES sysimage_stats (id) ON DELETE SET NULL;

ALTER TABLE sysimage_stats ADD CONSTRAINT sysimage_id_exists FOREIGN KEY (sysimage_id) REFERENCES sysimage (id) ON DELETE SET NULL;

ALTER TABLE system ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES system_stats (id) ON DELETE SET NULL;

ALTER TABLE system ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES system_stats (id) ON DELETE SET NULL;

ALTER TABLE system_stats ADD CONSTRAINT system_id_exists FOREIGN KEY (system_id) REFERENCES system (id) ON DELETE SET NULL;

ALTER TABLE vhost ADD CONSTRAINT stats_curr_id_exists FOREIGN KEY (stats_curr_id) REFERENCES vhost_stats (id) ON DELETE SET NULL;

ALTER TABLE vhost ADD CONSTRAINT stats_prev_id_exists FOREIGN KEY (stats_prev_id) REFERENCES vhost_stats (id) ON DELETE SET NULL;

ALTER TABLE vhost ADD CONSTRAINT broker_id_exists FOREIGN KEY (broker_id) REFERENCES broker (id) ON DELETE SET NULL;

ALTER TABLE vhost_stats ADD CONSTRAINT vhost_id_exists FOREIGN KEY (vhost_id) REFERENCES vhost (id) ON DELETE SET NULL;

